import os
import re
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
from ase.io import read

def curvef(loc, polymer, start, end, savefig, savedata, saveloc='./', xx=False):
    try:
        f = open(saveloc+'{0}_Result.txt'.format(polymer), 'x')
    except:
        f = open(saveloc+'{0}_Result.txt'.format(polymer), 'w')
    
    f.write('Strain Length(%),Stress Force(nN)\n')

    res = []
    for path in os.listdir(loc):
        if os.path.isfile(os.path.join(loc, path)):
            a = re.match(polymer+'_', path)
            if a:
                b = re.search('.xyz', path)
                if b:
                    c = re.search('trj', path)
                    if c:
                        continue
                    res.append(path)
    print(res)
    res.sort()
    
    try:
        atoms = read(loc+'{0}_0.000.xyz'.format(polymer))
    except:
        atoms = read(loc+'{0}_0.xyz'.format(polymer))
    x = atoms.get_distance(start, end)
    
    for data in res:
        atoms = read(loc+data)
        strain = re.search(r'_[0-9]+\.*[0-9]*', data)
        try:
            f.write('{0:.5f},{1}\n'.format(atoms.get_distance(start, end)/x,\
                                            strain.group(0)[1:]))
        except:
            continue
        y = atoms.get_distance(start, end)
    f.close()

    datum = pd.read_csv(saveloc+'{0}_Result.txt'.format(polymer))
    datum = datum[:].sort_values(by = ['Strain Length(%)']) #Sort document
    datum = datum[:-1]
    print(datum)
    if xx == False:
        a = sns.scatterplot(data=datum, x=datum['Strain Length(%)'],\
                            y=datum['Stress Force(nN)'], color='black', alpha=0.5)
        a.plot(datum['Strain Length(%)'], datum['Stress Force(nN)'])
        plt.xlabel('Strain Length(%)')
        plt.ylabel('Stress Force(nN)')
        _ = plt.title('Strain-Stress Curve')
    if savefig == True:
        plt.savefig(saveloc+'{0}.png'.format(polymer), bbox_inches = 'tight')
    if xx == False:
        plt.show()
    if savedata == False:
        os.system('rm {1}{0}_Result.txt'.format(polymer, saveloc))

def curvef2(loc, polymer, start, end, savefig, savedata, saveloc='./', xx=False):
    opath = os.getcwd()
    try:
        f = open(saveloc+'{0}_Result.txt'.format(polymer), 'x')
    except:
        f = open(saveloc+'{0}_Result.txt'.format(polymer), 'w')
    
    f.write('Force(nN),Distance(Å)\n')

    res = []
    for path in os.listdir(loc):
        if os.path.isfile(os.path.join(loc, path)):
            a = re.match(polymer+'_', path)
            if a:
                b = re.search('.xyz', path)
                if b:
                    c = re.search('trj', path)
                    if c:
                        continue
                    res.append(path)
    print(res)
    res.sort()
    
    try:
        atoms = read(loc+'{0}_0.000.xyz'.format(polymer))
    except:
        atoms = read(loc+'{0}_0.xyz'.format(polymer))
    x = atoms.get_distance(start, end)
    
    for data in res:
        atoms = read(loc+data)
        strain = re.search(r'_[0-9]+\.*[0-9]*', data)
        try:
            f.write('{0},{1:.5f}\n'.format(strain.group(0)[1:],\
                                            atoms.get_distance(start, end)-x))
        except:
            continue
        y = atoms.get_distance(start, end)
    f.close()

    datum = pd.read_csv(saveloc+'{0}_Result.txt'.format(polymer))
    datum = datum[:].sort_values(by=['Force(nN)']) #Sort document
    datum = datum[:-1]
    print(datum)
    if xx == False:
        a = sns.scatterplot(data=datum, x=datum['Force(nN)'],\
                            y=datum['Distance(Å)'], color='black', alpha=0.5)
        a.plot(datum['Force(nN)'], datum['Distance(Å)'])
        plt.xlabel('Force(nN)')
        plt.ylabel('Distance(Å)')
        _ = plt.title('Distance-Force Relationship')
    if savefig == True:
        plt.savefig(saveloc+'{0}.png'.format(polymer), bbox_inches='tight')
    if xx == False:
        plt.show()
    if savedata == False:
        os.system('rm {1}{0}_Result.txt'.format(polymer, saveloc))
    
class single:
    """
    The definition of polymer.
    single(polymer, loc='./')
    polymer: Your polymer name on your file.
    loc: File Location. The default is your current location.
    You can get the further information by .curve and .autocurve.
    """
    def __init__(self, polymer, loc='./'):
        self.loc = loc
        self.polymer = polymer

    def curve(self, start, end, savefig=True, savedata=True, saveloc='./', xx=False):
        """
    The method to create a single polymer chart of Stress-Strain Curve and .txt relevant data file of stress and strain.
    curve(start, end, savefig=True, savedata=True, saveloc='./')
    start, end: the beginning and end side of the polymer, mostly two sides imposing external force.
    savefig, savedata: To show if you want to save the chart and .txt curve. The default is True.
    saveloc: The location of your saved file. The default is your current location.
    The output of this file is a Stress-Strain Curve chart and relevant data.
    Example:
        Input:
            from MCPoly.sscurve import single
                poly1 = single('Polymer1')
                poly1.curve(0, 6)
        Output:
            ['Polymer1_2.0.xyz', 'Polymer1_0.0.xyz', 'Polymer1_0.5.xyz', 'Polymer1_2.5.xyz', 
             'Polymer1_1.0.xyz', 'Polymer1_3.0.xyz', 'Polymer1_1.5.xyz'] -> All your files
                  Strain Length(%)  Stress Force(nN)
                0           1.00000               0.0
                1           1.06778               0.5
                2           1.13094               1.0
                3           1.19338               1.5
                4           1.24726               2.0
                5           1.31094               2.5
            <matplotlib chart>
    TIPS: Make sure all your .xyz file is in the current location, or the chart may have some mistakes.
        """
        return curvef(self.loc, self.polymer, start, end, savefig, savedata, saveloc, xx)
    
    def autocurve(self, savefig=True, savedata=True, saveloc='./', xx=False):
        """
    The method to create a single polymer chart of Stress-Strain Curve and .txt relevant data file of stress and strain the same as function curve. but it can be more handy if there are some .out or .inp files. 
    autocurve(savefig=True, savedata=True, saveloc='./')
    savefig, savedata: To show if you want to save the chart and .txt curve. The default is True.
    saveloc: The location of your saved file. The default is your current location.
    The output of this file is a Stress-Strain Curve chart and relevant data.
    Example:
        Input:
            from MCPoly.sscurve import single
                poly1 = single('Polymer1')
                poly1.autocurve()
        Output:
            ['Polymer1_2.0.xyz', 'Polymer1_0.0.xyz', 'Polymer1_0.5.xyz', 'Polymer1_2.5.xyz', 
             'Polymer1_1.0.xyz', 'Polymer1_3.0.xyz', 'Polymer1_1.5.xyz']
                  Strain Length(%)  Stress Force(nN)
                0           1.00000               0.0
                1           1.06778               0.5
                2           1.13094               1.0
                3           1.19338               1.5
                4           1.24726               2.0
                5           1.31094               2.5
            <matplotlib chart>
    TIPS: Make sure all your .xyz file is in the current location, or the chart may have some mistakes.
          Make sure there is at least one .out or .inp file of relevant polymer here, or it will show FileNotFoundError.
        """
        t = 0
        for path in os.listdir(self.loc):
            if os.path.isfile(os.path.join(self.loc, path)):
                a = re.match(self.polymer+'_', path)
                if a:
                    b = re.search('.inp', path)
                    if b:
                        t = 1
                        break
                    c = re.search('.out', path)
                    if c:
                        t = 2
                        break
        if t == 0:
            raise FileNotFoundError('relevant .inp and .out file is not found.\
                Try to find out the file and put it in this document.')
        else:
            opath = os.getcwd()
            os.chdir(self.loc)
            f = open(path, 'r')
            i = 0
            start = -1
            end = -1
            if t == 1:
                for line in f:
                    if i == 1:
                        num = re.findall(r'[0-9]+', line)
                        start = eval(num[0])
                        end = eval(num[1])
                        break
                    a = re.search('POTENTIALS', line)
                    if a:
                        i = i + 1
            if t == 2:
                for line in f:
                    if i == 1:
                        num = re.findall(r'[0-9]+', line)
                        start = eval(num[1])
                        end = eval(num[2])
                        break
                    a = re.search('> POTENTIALS', line)
                    if a:
                        i = i + 1
            os.chdir(opath)
        return curvef(self.loc, self.polymer, start, end, savefig, savedata, saveloc, xx)

    def curve2(self, start, end, savefig=True, savedata=True, saveloc='./', xx=False):
        """
    The method to create a single polymer chart of Distance-Force Curve and .txt relevant data file of stress and strain.
    curve(start, end, savefig = True, savedata = True, saveloc='./')
    start, end: the beginning and end side of the polymer, mostly two sides imposing external force.
    savefig, savedata: To show if you want to save the chart and .txt curve. The default is True.
    saveloc: The location of your saved file. The default is your current location.
    The output of this file is a Distance-Force Curve chart and relevant data.
    Example:
        Input:
            from MCPoly.sscurve import single
                poly1 = single('Polymer1')
                poly1.curve2(0, 6)
        Output:
            ['Polymer1_2.0.xyz', 'Polymer1_0.0.xyz', 'Polymer1_0.5.xyz', 'Polymer1_2.5.xyz', 
             'Polymer1_1.0.xyz', 'Polymer1_3.0.xyz', 'Polymer1_1.5.xyz'] -> All your files
                  Distance(Å)       Force(nN)
                0           1.00000               0.0
                1           1.06778               0.5
                2           1.13094               1.0
                3           1.19338               1.5
                4           1.24726               2.0
                5           1.31094               2.5
            <matplotlib chart>
    TIPS: Make sure all your .xyz file is in the current location, or the chart may have some mistakes.
        """
        return curvef2(self.loc, self.polymer, start, end, savefig, savedata, saveloc, xx)
    
    def autocurve2(self, savefig=True, savedata=True, saveloc='./', xx=False):
        """
    The method to create a single polymer chart of Distance-Force Curve and .txt relevant data file of stress and strain the same as function curve. but it can be more handy if there are some .out or .inp files. 
    autocurve(savefig=True, savedata=True, saveloc='./')
    savefig, savedata: To show if you want to save the chart and .txt curve. The default is True.
    saveloc: The location of your saved file. The default is your current location.
    The output of this file is a Distance-Force Curve chart and relevant data.
    Example:
        Input:
            from MCPoly.sscurve import single
                poly1 = single('Polymer1')
                poly1.autocurve2()
        Output:
            ['Polymer1_2.0.xyz', 'Polymer1_0.0.xyz', 'Polymer1_0.5.xyz', 'Polymer1_2.5.xyz', 
             'Polymer1_1.0.xyz', 'Polymer1_3.0.xyz', 'Polymer1_1.5.xyz']
                  Distance(Å)       Force(nN)
                0           1.00000               0.0
                1           1.06778               0.5
                2           1.13094               1.0
                3           1.19338               1.5
                4           1.24726               2.0
                5           1.31094               2.5
            <matplotlib chart>
    TIPS: Make sure all your .xyz file is in the current location, or the chart may have some mistakes.
          Make sure there is at least one .out or .inp file of relevant polymer here, or it will show FileNotFoundError.
        """
        t = 0
        for path in os.listdir(self.loc):
            if os.path.isfile(os.path.join(self.loc, path)):
                a = re.match(self.polymer+'_', path)
                if a:
                    b = re.search('.inp', path)
                    if b:
                        t = 1
                        break
                    c = re.search('.out', path)
                    if c:
                        t = 2
                        break
        if t == 0:
            raise FileNotFoundError('relevant .inp and .out file is not found. Try to find out the file and put it in this document.')
        else:
            opath = os.getcwd()
            os.chdir(self.loc)
            f = open(path, 'r')
            i = 0
            start = -1
            end = -1
            if t == 1:
                for line in f:
                    if i == 1:
                        num = re.findall(r'[0-9]+', line)
                        start = eval(num[0])
                        end = eval(num[1])
                        break
                    a = re.search('POTENTIALS', line)
                    if a:
                        i = i + 1
            if t == 2:
                for line in f:
                    if i == 1:
                        num = re.findall(r'[0-9]+', line)
                        start = eval(num[1])
                        end = eval(num[2])
                        break
                    a = re.search('> POTENTIALS', line)
                    if a:
                        i = i + 1
            os.chdir(opath)
        return curvef2(self.loc, self.polymer, start, end, savefig, savedata, saveloc, xx)


import math as m
import numpy as np
from ase.io import read, write
from ase import Atoms
from ase.visualize import view

def C_selection(substitution, atoms, Cnear, state=[1, 1]):
    if substitution in ('CO', 'C=O'):
        return CO(atoms, Cnear, state)
    if substitution in ('CNH', 'C=NH'):
        return CNH(atoms, Cnear, state)
    if substitution == 'O':
        return O(atoms, Cnear, state)
    if substitution == 'S':
        return S(atoms, Cnear, state)
    if substitution == 'N':
        return N(atoms, Cnear, state)
                        
def CO(atoms, Cnear, state=[1, 1]):
    atoms.append('O')
    total_atoms = len(atoms)
    x = atoms.get_positions()
    x[-1][0] = x[-1][0] - 50
    x[-1][1] = x[-1][1] - 50
    x[-1][2] = x[-1][2] - 50
    atoms.set_positions(x)
    CnearC1 = near(Cnear, atoms, 'C')
    CnearC2 = near(Cnear, atoms, 'C', ignore=[CnearC1], reverse=True)
    #print(CnearC1, CnearC2)
    mask = [0]*(total_atoms-1)+[1]
    mask[Cnear] = 1
    for k in range(10):
        if state[0] == 1:
            atoms.set_distance(Cnear, CnearC1, 1.5220, fix=1, mask=mask)
        elif state[0] == 2:
            atoms.set_distance(Cnear, CnearC1, 1.4440, fix=1, mask=mask)
        elif state[0] == 6:
            atoms.set_distance(Cnear, CnearC1, 1.4900, fix=1, mask=mask)
        elif state[0] == 7:
            atoms.set_distance(Cnear, CnearC1, 1.3350, fix=1, mask=mask)
        elif state[0] == 8:
            atoms.set_distance(Cnear, CnearC1, 1.3640, fix=1, mask=mask)
        elif state[0] == 82:
            atoms.set_distance(Cnear, CnearC1, 1.5100, fix=1, mask=mask)
        if state[1] == 1:
            atoms.set_distance(Cnear, CnearC2, 1.5220, fix=1, mask=mask)
        elif state[1] == 2:
            atoms.set_distance(Cnear, CnearC2, 1.4440, fix=1, mask=mask)
        elif state[1] == 6:
            atoms.set_distance(Cnear, CnearC2, 1.4900, fix=1, mask=mask)
        elif state[1] == 7:
            atoms.set_distance(Cnear, CnearC2, 1.3350, fix=1, mask=mask)
        elif state[1] == 8:
            atoms.set_distance(Cnear, CnearC2, 1.3640, fix=1, mask=mask)
        elif state[1] == 82:
            atoms.set_distance(Cnear, CnearC2, 1.5100, fix=1, mask=mask)
    mask[Cnear] = 0
    for k in range(10):
        atoms.set_angle(CnearC1, Cnear, total_atoms-1, 120, mask=mask)
        atoms.set_angle(CnearC2, Cnear, total_atoms-1, 120, mask=mask)
    atoms.set_distance(total_atoms-1, Cnear, 1.2290, fix=1, mask=mask)
    for i in range(2):
        Hnear = near(Cnear, atoms, 'H', limit = False)
        del atoms[Hnear]
    return atoms

def CNH(atoms, Cnear, state=[1, 1]):
    atoms.append('N')
    atoms.append('H')
    total_atoms = len(atoms)
    x = atoms.get_positions()
    x[-2][0] = x[-2][0] + 50
    x[-2][1] = x[-2][1] + 50
    x[-2][2] = x[-2][2] + 50
    x[-1][0] = x[-1][0] - 50
    x[-1][1] = x[-1][1] - 50
    x[-1][2] = x[-1][2] - 50
    atoms.set_positions(x)
    CnearC1 = near(Cnear, atoms, 'C')
    CnearC2 = near(Cnear, atoms, 'C', ignore=[CnearC1], reverse=True)
    #print(CnearC1, CnearC2)
    mask = [0]*(total_atoms-2) + [2]
    mask[Cnear] = 1
    for k in range(10):
        if state[0] == 1:
            atoms.set_distance(Cnear, CnearC1, 1.5220, fix=1, mask=mask)
        elif state[0] == 2:
            atoms.set_distance(Cnear, CnearC1, 1.5220, fix=1, mask=mask)
        elif state[0] == 6:
            atoms.set_distance(Cnear, CnearC1, 1.5220, fix=1, mask=mask)
        elif state[0] == 7:
            atoms.set_distance(Cnear, CnearC1, 1.3350, fix=1, mask=mask)
        elif state[0] == 8:
            atoms.set_distance(Cnear, CnearC1, 1.3640, fix=1, mask=mask)
        elif state[0] == 82:
            atoms.set_distance(Cnear, CnearC1, 1.5100, fix=1, mask=mask)
        if state[1] == 1:
            atoms.set_distance(Cnear, CnearC2, 1.5220, fix=1, mask=mask)
        elif state[1] == 2:
            atoms.set_distance(Cnear, CnearC2, 1.5220, fix=1, mask=mask)
        elif state[1] == 6:
            atoms.set_distance(Cnear, CnearC2, 1.5220, fix=1, mask=mask)
        elif state[1] == 7:
            atoms.set_distance(Cnear, CnearC2, 1.3350, fix=1, mask=mask)
        elif state[1] == 8:
            atoms.set_distance(Cnear, CnearC2, 1.3640, fix=1, mask=mask)
        elif state[1] == 82:
            atoms.set_distance(Cnear, CnearC2, 1.5100, fix=1, mask=mask)
    mask[Cnear] = 0
    for k in range(10):
        atoms.set_angle(CnearC1, Cnear, total_atoms-2, 120, mask=mask)
        atoms.set_angle(CnearC2, Cnear, total_atoms-2, 120, mask=mask)
    mask[-2] = 0
    atoms.set_angle(Cnear, total_atoms-2, total_atoms-1, 120.5, mask=mask)
    atoms.set_distance(total_atoms-1, total_atoms-2, 1.0100, fix=1, mask=mask)
    atoms.set_dihedral(Cnear1, Cnear, total_atoms-2,\
                       total_atoms-1, 179.5, mask=mask)
    for i in range(2):
        Hnear = near(Cnear, atoms, 'H', limit = False)
        del atoms[Hnear]
    return atoms

def O(atoms, Cnear, state=[1, 1]):
    atoms.append('O')
    total_atoms = len(atoms)
    x = atoms.get_positions()
    x[-1][0] = x[-1][0] - 50
    x[-1][1] = x[-1][1] - 50
    x[-1][2] = x[-1][2] - 50
    atoms.set_positions(x)
    CnearC1 = near(Cnear, atoms, 'C')
    CnearC2 = near(Cnear, atoms, 'C', ignore=[CnearC1], reverse=True)
    print(CnearC1, CnearC2)
    mask = [0]*(total_atoms-1) + [1]
    atoms.set_distance(total_atoms-1, Cnear, 0, fix=1)
    for k in range(10):
        if state[0] == 1:
            atoms.set_distance(total_atoms-1, CnearC1, 1.410, fix=1)
        elif state[0] == 2 or state[0] == 6 or state[0] == 82:
            atoms.set_distance(total_atoms-1, CnearC1, 1.364, fix=1)
        elif state[0] == 4:
            atoms.set_distance(total_atoms-1, CnearC1, 1.327, fix=1)
        elif state[0] == 164:
            atoms.set_distance(total_atoms-1, CnearC1, 1.670, fix=1)
        if state[1] == 1:
            atoms.set_distance(total_atoms-1, CnearC2, 1.410, fix=1)
        elif state[1] == 2 or state[1] == 6 or state[1] == 82:
            atoms.set_distance(total_atoms-1, CnearC2, 1.364, fix=1)
        elif state[1] == 4:
            atoms.set_distance(total_atoms-1, CnearC2, 1.327, fix=1)
        elif state[1] == 164:
            atoms.set_distance(total_atoms-1, CnearC2, 1.670, fix=1)
    del atoms[Cnear]
    for i in range(2):
        total_atoms = len(atoms)
        Hnear = near(total_atoms-1, atoms, 'H', limit=False)
        del atoms[Hnear]
    return atoms

def S(atoms, Cnear, state=[1, 1]):
    atoms.append('S')
    total_atoms = len(atoms)
    x = atoms.get_positions()
    x[-1][0] = x[-1][0] - 50
    x[-1][1] = x[-1][1] - 50
    x[-1][2] = x[-1][2] - 50
    atoms.set_positions(x)
    CnearC1 = near(Cnear, atoms, 'C')
    CnearC2 = near(Cnear, atoms, 'C', ignore=[CnearC1], reverse=True)
    print(CnearC1, CnearC2)
    mask = [0]*(total_atoms-1) + [1]
    atoms.set_distance(total_atoms-1, Cnear, 0, fix=1)
    for k in range(10):
        if state[0] == 1:
            atoms.set_distance(total_atoms-1, CnearC1, 1.810, fix=1)
        elif state[0] == 2:
            atoms.set_distance(total_atoms-1, CnearC1, 1.770, fix=1)
        if state[1] == 1:
            atoms.set_distance(total_atoms-1, CnearC2, 1.810, fix=1)
        elif state[1] == 2:
            atoms.set_distance(total_atoms-1, CnearC2, 1.770, fix=1)
    Hnear = near(total_atoms-1, atoms, 'C', limit=False)
    del atoms[Hnear]
    for i in range(2):
        total_atoms = len(atoms)
        Hnear = near(total_atoms-1, atoms, 'H', limit=False)
        del atoms[Hnear]
    return atoms

def N(atoms, Cnear, state=[1, 1]):
    atoms.append('N')
    total_atoms = len(atoms)
    x = atoms.get_positions()
    x[-1][0] = x[-1][0] - 50
    x[-1][1] = x[-1][1] - 50
    x[-1][2] = x[-1][2] - 50
    atoms.set_positions(x)
    CnearC1 = near(Cnear, atoms, 'C')
    CnearC2 = near(Cnear, atoms, 'C', ignore=[CnearC1], reverse=True)
    print(CnearC1, CnearC2)
    mask = [0]*(total_atoms-1) + [1]
    atoms.set_distance(total_atoms-1, Cnear, 0, fix=1)
    for k in range(10):
        if state[0] == 1:
            atoms.set_distance(total_atoms-1, CnearC1, 1.4480, fix=1)
        elif state[0] == 2 or state[0] == 6 or state[0] == 82:
            atoms.set_distance(total_atoms-1, CnearC1, 1.3400, fix=1)
        elif state[0] == 4:
            atoms.set_distance(total_atoms-1, CnearC1, 1.4900, fix=1)
        if state[1] == 1:
            atoms.set_distance(total_atoms-1, CnearC2, 1.4480, fix=1)
        elif state[1] == 2 or state[1] == 6 or state[1] == 82:
            atoms.set_distance(total_atoms-1, CnearC2, 1.3400, fix=1)
        elif state[1] == 4:
            atoms.set_distance(total_atoms-1, CnearC2, 1.4900, fix=1)
    CnearC1 = numjudge(CnearC1, Cnear)
    CnearC2 = numjudge(CnearC2, Cnear)
    del atoms[Cnear]
    for i in range(2):
        total_atoms = len(atoms)
        Hnear = near(total_atoms-1, atoms, 'H', limit=False)
        del atoms[Hnear]
        CnearC1 = numjudge(CnearC1, Hnear)
        CnearC2 = numjudge(CnearC2, Hnear)
    atoms.append('H')
    total_atoms = len(atoms)
    mask = [0]*(total_atoms-1) + [1]
    CnearC3 = near(CnearC1, atoms, 'C', ignore = [CnearC1, CnearC2])
    for k in range(10):
        atoms.set_angle(CnearC1, total_atoms-2, total_atoms-1, 109.5, mask=mask)
        atoms.set_angle(CnearC2, total_atoms-2, total_atoms-1, 109.5, mask=mask)
        atoms.set_dihedral(CnearC3, CnearC1, total_atoms-2,\
                           total_atoms-1, 150, mask=mask)
        atoms.set_distance(total_atoms-1, total_atoms-2, 1.0100, fix=1)
    return atoms


import math as m
import numpy as np
from ase.io import read, write
from ase import Atoms
from ase.visualize import view

def bind_selection(ring_size, atoms, bond, state=[1, 1], place=0, strict={1:'X'}, side=0, shuffle=0, less5=False, stable=False):
    '''This function will bind the ring onto a bond. 5 means adding a 5-membered-ring bound. 
    Adding a 6 in the front will make sure the conjugation of the ring.''' 
    
    if ring_size in (2, 'bind2'):
        return bind2(atoms, bond, state)
    elif ring_size in (3, 'bind3'):
        return bind3(atoms, bond, state, strict)
    elif ring_size in (5, 'bind5'):
        return bind5(atoms, bond, state, strict, side, shuffle, less5, stable)
    elif ring_size (6, 'bind6'):
        return bind6(atoms, bond, state, strict, side, shuffle, less5)
    elif ring_size (65, 'bindconj5'):
        return bindconj5(atoms, bond, state, strict)
    elif ring_size (66, 'bindconj6'):
        return bindconj6(atoms, bond, state, strict)

def bind2(atoms, bond, state=[1, 1]):
    i1 = bond[0]
    i2 = bond[1]
    elements = atoms.get_chemical_symbols()
    element1 = elements[i1]
    element2 = elements[i2]
    Cnear1 = near(i1, atoms, 'Not H', ignore=[i1, i2])
    Cnear2 = near(i2, atoms, 'Not H', ignore=[i1, i2], reverse=True)
    total_atoms = len(atoms)
    mask = [0] * total_atoms
    mask[i1] = 1
    mask[i2] = 1
    bondjudge2([Cnear1, Cnear2], atoms, [i1, i2], [element1, element2],\
               state, mask)
    for j in range(2):
        CnearH = near(i1, atoms, 'H')
        del atoms[CnearH]
        i1 = numjudge(i1, CnearH)
        i2 = numjudge(i2, CnearH)
    for j in range(2):
        CnearH = near(i2, atoms, 'H')
        del atoms[CnearH]
        i1 = numjudge(i1, CnearH)
        i2 = numjudge(i2, CnearH)
    atoms.append('H')
    atoms.append('H')
    x = atoms.get_positions()
    x[-2][0] = x[-2][0] - 100
    x[-2][1] = x[-2][1] - 50
    x[-2][2] = x[-2][2] - 50
    x[-1][0] = x[-1][0] + 50
    x[-1][1] = x[-1][1] + 50
    x[-1][2] = x[-1][2] + 50
    atoms.set_positions(x)
    total_atoms = len(atoms)
    mask = [0]*(total_atoms-2) + [1]*2
    for k in range(10):
        atoms.set_angle(Cnear1, i1, total_atoms-2, 118, mask=mask)
        atoms.set_angle(i2, i1, total_atoms-2, 118, mask=mask)
    atoms.set_distance(total_atoms-2, i1, 1.0800, fix=1, mask=mask)
    mask[-2] = 0
    for k in range(10):
        atoms.set_angle(Cnear2, i2, total_atoms-1, 118, mask=mask)
        atoms.set_angle(i1, i2, total_atoms-1, 118, mask=mask)
    atoms.set_distance(total_atoms-1, i2, 1.0800, fix=1, mask=mask)
    atoms.set_dihedral(total_atoms-2, i1, i2, total_atoms-1, 0.5, mask=mask)
    if element1 == 'N':
        del atoms[-2]
    if element2 == 'N':
        del atoms[-1]
    return atoms

def bind3(atoms, bond, state=[1, 1], strict={1:'X'}):
    i1 = bond[0]
    i2 = bond[1]
    Cnear = near(i1, atoms, 'C')
    total_atoms = len(atoms)
    if strict[1] == 'O':
        atoms.append('O')
        mask = [0]*total_atoms + [1]
        if state == [1, 1]:
            for k in range(10):
                atoms.set_distance(total_atoms, i1, 1.4100, fix=1, mask=mask)
                atoms.set_distance(total_atoms, i2, 1.4100, fix=1, mask=mask)
        elif state == [1, 2]:
            for k in range(10):
                atoms.set_distance(total_atoms, i1, 1.4100, fix=1, mask=mask)
                atoms.set_distance(total_atoms, i2, 1.3640, fix=1, mask=mask)
        elif state == [2, 1]:
            for k in range(10):
                atoms.set_distance(total_atoms, i1, 1.3640, fix=1, mask=mask)
                atoms.set_distance(total_atoms, i2, 1.4100, fix=1, mask=mask)
        elif state == [2, 2] or state == [6, 6]:
            for k in range(10):
                atoms.set_distance(total_atoms, i1, 1.3640, fix=1, mask=mask)
                atoms.set_distance(total_atoms, i2, 1.3640, fix=1, mask=mask)
        atoms.set_dihedral(Cnear, i1, i2, total_atoms, 75, mask=mask)
        for j in range(2):
            CnearH = near(total_atoms - j, atoms, 'H')
            del atoms[CnearH]
    elif strict[1] == 'S':
        atoms.append('S')
        mask = [0]*total_atoms + [1]
        if state == [1, 1]:
            for k in range(10):
                atoms.set_distance(total_atoms, i1, 1.8100, fix=1, mask=mask)
                atoms.set_distance(total_atoms, i2, 1.8100, fix=1, mask=mask)
        elif state == [1, 2]:
            for k in range(10):
                atoms.set_distance(total_atoms, i1, 1.8100, fix=1, mask=mask)
                atoms.set_distance(total_atoms, i2, 1.7700, fix=1, mask=mask)
        elif state == [2, 1]:
            for k in range(10):
                atoms.set_distance(total_atoms, i1, 1.7700, fix=1, mask=mask)
                atoms.set_distance(total_atoms, i2, 1.8100, fix=1, mask=mask)
        elif state == [2, 2] or state == [6, 6]:
            for k in range(10):
                atoms.set_distance(total_atoms, i1, 1.7700, fix=1, mask=mask)
                atoms.set_distance(total_atoms, i2, 1.7700, fix=1, mask=mask)
        atoms.set_dihedral(Cnear, i1, i2, total_atoms, 75, mask=mask)
        for j in range(2):
            CnearH = near(total_atoms, atoms, 'H')
            del atoms[CnearH]
            total_atoms = numjudge(total_atoms, CnearH)
            i1 = numjudge(i1, CnearH)
            i2 = numjudge(i2, CnearH)
    elif strict[1] == 'N':
        atoms.append('N')
        atoms.append('H')
        mask = [0]*total_atoms + [1]*2
        if state == [1, 1]:
            for k in range(10):
                atoms.set_distance(total_atoms, i1, 1.4480, fix=1, mask=mask)
                atoms.set_distance(total_atoms, i2, 1.4480, fix=1, mask=mask)
        elif state == [1, 2]:
            for k in range(10):
                atoms.set_distance(total_atoms, i1, 1.4480, fix=1, mask=mask)
                atoms.set_distance(total_atoms, i2, 1.3400, fix=1, mask=mask)
        elif state == [2, 1]:
            for k in range(10):
                atoms.set_distance(total_atoms, i1, 1.3400, fix=1, mask=mask)
                atoms.set_distance(total_atoms, i2, 1.4480, fix=1, mask=mask)
        elif state == [2, 2] or state == [6, 6]:
            for k in range(10):
                atoms.set_distance(total_atoms, i1, 1.3400, fix=1, mask=mask)
                atoms.set_distance(total_atoms, i2, 1.3400, fix=1, mask=mask)
        atoms.set_dihedral(Cnear, i1, i2, total_atoms, 75, mask=mask)
        mask[-2] = 0
        for j in range(2):
            CnearH = near(total_atoms, atoms, 'H')
            del atoms[CnearH]
            total_atoms = numjudge(total_atoms, CnearH)
            i1 = numjudge(i1, CnearH)
            i2 = numjudge(i2, CnearH)
        x = atoms.get_positions()
        x[-1][0] = x[-1][0] + 50
        x[-1][1] = x[-1][1] + 50
        x[-1][2] = x[-1][2] + 50
        atoms.set_positions(x)
        total_atoms = len(atoms)
        mask = [0]*(total_atoms-1) + [1]
        atoms.set_distance(total_atoms-1, total_atoms-2, 0, fix=1, mask=mask)
        atoms.set_distance(total_atoms-1, total_atoms-2, 1.0100, fix=1, mask=mask)
        atoms.set_dihedral(total_atoms-1, total_atoms-2, i1, i2, -90.5, mask=mask)
        for k in range(10):
            atoms.set_angle(i1, total_atoms-2, total_atoms-1, 111.5, mask=mask)
            atoms.set_angle(i2, total_atoms-2, total_atoms-1, 111.5, mask=mask)
    else:
        atoms.append('C')
        atoms.append('H')
        atoms.append('H')
        mask = [0]*total_atoms + [1]*3
        if state == [1, 1]:
            for k in range(10):
                atoms.set_distance(total_atoms, i1, 1.5290, fix=1, mask=mask)
                atoms.set_distance(total_atoms, i2, 1.5290, fix=1, mask=mask)
        elif state == [1, 2]:
            for k in range(10):
                atoms.set_distance(total_atoms, i1, 1.5290, fix=1, mask=mask)
                atoms.set_distance(total_atoms, i2, 1.5095, fix=1, mask=mask)
        elif state == [2, 1]:
            for k in range(10):
                atoms.set_distance(total_atoms, i1, 1.5095, fix=1, mask=mask)
                atoms.set_distance(total_atoms, i2, 1.5290, fix=1, mask=mask)
        elif state == [2, 2]:
            for k in range(10):
                atoms.set_distance(total_atoms, i1, 1.5095, fix=1, mask=mask)
                atoms.set_distance(total_atoms, i2, 1.5095, fix=1, mask=mask)
        elif state == [6, 6]:
            for k in range(10):
                atoms.set_distance(total_atoms, i1, 1.5105, fix=1, mask=mask)
                atoms.set_distance(total_atoms, i2, 1.5105, fix=1, mask=mask)
        atoms.set_dihedral(Cnear, i1, i2, total_atoms, 75, mask=mask)
        mask[-3] = 0
        for j in range(2):
            CnearH = near(total_atoms, atoms, 'H')
            del atoms[CnearH]
            total_atoms = numjudge(total_atoms, CnearH)
            i1 = numjudge(i1, CnearH)
            i2 = numjudge(i2, CnearH)
        x = atoms.get_positions()
        x[-2][0] = x[-2][0] + 50
        x[-2][1] = x[-2][1] + 50
        x[-2][2] = x[-2][2] + 50
        x[-1][0] = x[-1][0] - 50
        x[-1][1] = x[-1][1] - 50
        x[-1][2] = x[-1][2] - 50
        atoms.set_positions(x)
        total_atoms = len(atoms)
        mask = [0]*(total_atoms-2) + [1]*2
        atoms.set_distance(total_atoms-2, total_atoms-3, 0, fix=1, mask=mask)
        atoms.set_distance(total_atoms-2, total_atoms-3, 1.0900, fix=1, mask=mask)
        atoms.set_dihedral(total_atoms-2, total_atoms-3, i1, Cnear, 110, mask=mask)
        for k in range(10):
            atoms.set_angle(i1, total_atoms-3, total_atoms-2, 111.5, mask=mask)
            atoms.set_angle(i2, total_atoms-3, total_atoms-2, 111.5, mask=mask)
        mask[-2] = 0
        atoms.set_distance(total_atoms-1, total_atoms-3, 0, fix=1, mask=mask)
        atoms.set_distance(total_atoms-1, total_atoms-3, 1.0900, fix=1, mask=mask)
        atoms.set_dihedral(total_atoms-1, total_atoms-3, i1, Cnear, -70.5, mask=mask)
        for k in range(10):
            atoms.set_angle(i1, total_atoms-3, total_atoms-1, 111.5, mask=mask)
            atoms.set_angle(i2, total_atoms-3, total_atoms-1, 111.5, mask=mask)
        atoms.set_dihedral(total_atoms-1, total_atoms-3, i1, i2, -90.5, mask=mask)
    return atoms

def bind5(atoms, bond, state=[1, 1], strict={1:'X'}, side=0, shuffle=0, less5=False, stable=False):
    i1 = bond[0]
    i2 = bond[1]
    if side == 0:
        Hnear1 = near(i1, atoms, 'H')
        Hnear2 = near(i2, atoms, 'H')
    elif side == 1:
        Hnear1 = near(i1, atoms, 'H', reverse=True)
        Hnear2 = near(i2, atoms, 'H', reverse=True)
    elif side == 2:
        Hnear1 = near(i1, atoms, 'H')
        Hnear2 = near(i2, atoms, 'H', reverse=True)
    elif side == 3:
        Hnear1 = near(i1, atoms, 'H', reverse=True)
        Hnear2 = near(i2, atoms, 'H')
    print(Hnear1, Hnear2)
    total_atoms = len(atoms)
    mask = [0] * total_atoms
    mask[Hnear1] = 1
    atoms.set_angle(i2, i1, Hnear1, 100, mask=mask)
    mask[Hnear1] = 0
    mask[Hnear2] = 1
    atoms.set_angle(i1, i2, Hnear2, -100, mask=mask)
    if (less5 == True and side < 2) or (less5 == False and side >= 2):
        atoms.set_dihedral(Hnear1, i1, i2, Hnear2, -35, mask=mask)
        xoy = near(Hnear2, atoms, limit=False)
        if atoms.get_distance(xoy, Hnear2) <= 0.9:
            atoms.set_dihedral(Hnear1, i1, i2, Hnear2, 35, mask=mask)
        xoy = near(i2, atoms, target='Not H')
        if xoy == i1:
            xoy = near(i2, atoms, target='Not H', reverse=True)
        if atoms.get_angle(xoy, i2, Hnear2) <= 90:
            atoms.set_dihedral(Hnear1, i1, i2, Hnear2, 35, mask=mask)
    elif less5 == False and side < 2:
        atoms.set_dihedral(Hnear1, i1, i2, Hnear2, 55, mask=mask)
        xoy = near(Hnear2, atoms, limit=False)
        if atoms.get_distance(xoy, Hnear2) <= 0.9:
            atoms.set_dihedral(Hnear1, i1, i2, Hnear2, -55, mask=mask)
        xoy = near(i2, atoms, target='Not H')
        if xoy == i1:
            xoy = near(i2, atoms, target='Not H', reverse=True)
        if atoms.get_angle(xoy, i2, Hnear2) <= 90:
            atoms.set_dihedral(Hnear1, i1, i2, Hnear2, -55, mask=mask)
    elif less5 == True and side >= 2:
        atoms.set_dihedral(Hnear1, i1, i2, Hnear2, 80, mask=mask)
        xoy = near(Hnear2, atoms, limit=False)
        if atoms.get_distance(xoy, Hnear2) <= 0.9:
            atoms.set_dihedral(Hnear1, i1, i2, Hnear2, -80, mask=mask)
        xoy = near(i2, atoms, target='Not H')
        if xoy == i1:
            xoy = near(i2, atoms, target='Not H', reverse=True)
        if atoms.get_angle(xoy, i2, Hnear2) <= 90:
            atoms.set_dihedral(Hnear1, i1, i2, Hnear2, -80, mask=mask)
        mask[Hnear2] = 0
        mask[Hnear1] = 1
        if atoms.get_dihedral(Hnear1, i1, i2, Hnear2)<85:
            atoms.set_dihedral(Hnear2, i2, i1, Hnear1, 50, mask=mask)
        else:
            atoms.set_dihedral(Hnear2, i2, i1, Hnear1, 360 - 50, mask=mask)
    if less5 == True and side >= 2:
        elements = atoms.get_chemical_symbols()
        elements[Hnear1] = 'C'
        elements[Hnear2] = 'C'
        atoms.set_chemical_symbols(elements)
        atoms.set_distance(Hnear1, i1, 1.0400, fix=1)
        atoms.set_distance(Hnear2, i2, 1.0400, fix=1)
        Hnear3 = near(i1, atoms, 'H')
        Hnear4 = near(i2, atoms, 'H')
        atoms.set_distance(Hnear1, i1, 1.0900, fix=1)
        atoms.set_distance(Hnear2, i2, 1.0900, fix=1)
        elements[Hnear1] = 'H'
        elements[Hnear2] = 'H'
        atoms.set_chemical_symbols(elements)
    #    x = atoms.get_positions()
    #    x[Hnear3][0] = x[Hnear3][0] - 50
    #    x[Hnear3][1] = x[Hnear3][1] - 50
    #    x[Hnear3][2] = x[Hnear3][2] - 50
    #    x[Hnear4][0] = x[Hnear4][0] + 50
    #    x[Hnear4][1] = x[Hnear4][1] + 50
    #    x[Hnear4][2] = x[Hnear4][2] + 50
    #    atoms.set_positions(x)
    if less5 == True and side >= 2:
        Cnear1 = near(i1, atoms, 'Not H')
        Cnear2 = near(i2, atoms, 'Not H')
        mask[Hnear1] = 0
        mask[Hnear2] = 0
        mask[Hnear3] = 1
        mask[Hnear4] = 1
        if Cnear1 == i2:
            Cnear1 = near(i1, atoms, 'Not H', reverse=True)
        if Cnear2 == i1:
            Cnear2 = near(i2, atoms, 'Not H', reverse=True)
        for k in range(10):
            atoms.set_angle(i2, i1, Hnear3, 105.0, mask=mask)
            atoms.set_angle(Hnear1, i1, Hnear3, 105.0, mask=mask)
        atoms.set_distance(Hnear3, i1, 1.0900, fix=1, mask=mask)
        mask[Hnear3] = 0
        for k in range(10):
            atoms.set_angle(i1, i2, Hnear4, 105.0, mask=mask)
            atoms.set_angle(Hnear2, i2, Hnear4, 105.0, mask=mask)
        atoms.set_distance(Hnear4, i2, 1.0900, fix=1, mask=mask)
    if shuffle == 0:
        atoms = CH3(atoms, Hnear1, state[0])
        atoms = CH3(atoms, Hnear2, state[1], ring=True)
    elif shuffle == 1:
        atoms = CH3(atoms, Hnear1, state[0], ring=True)
        atoms = CH3(atoms, Hnear2, state[1])
    elif shuffle == 2:
        atoms = CH3(atoms, Hnear1, state[0])
        atoms = CH3(atoms, Hnear2, state[1])
    elif shuffle == 3:
        atoms = CH3(atoms, Hnear1, state[0], ring=True)
        atoms = CH3(atoms, Hnear2, state[1], ring=True)
    atoms.append('C')
    atoms.append('H')
    atoms.append('H')
    x = atoms.get_positions()
    x[-3][0] = x[-3][0] - 50
    x[-3][1] = x[-3][1] - 50
    x[-3][2] = x[-3][2] - 50
    x[-2][0] = x[-2][0] + 50
    x[-2][1] = x[-2][1] + 50
    x[-2][2] = x[-2][2] + 50
    x[-1][0] = x[-1][0] - 50
    x[-1][1] = x[-1][1] - 50
    x[-1][2] = x[-1][2] - 50
    atoms.set_positions(x)
    total_atoms = len(atoms)
    mask = [0]*(total_atoms-3)+[1]*3
    for k in range(10):
        if atoms.get_distance(total_atoms-3, i1) < 1.6 or stable == True:
            atoms.set_distance(total_atoms-3, i1, 2.4, fix=1, mask=mask)
        if atoms.get_distance(total_atoms-3, i2) < 1.6 or stable == True:
            atoms.set_distance(total_atoms-3, i2, 2.4, fix=1, mask=mask)
        atoms.set_distance(total_atoms-3, Hnear1, 1.5290, fix=1, mask=mask)
        atoms.set_distance(total_atoms-3, Hnear2, 1.5290, fix=1, mask=mask)
    mask[-3] = 0
    for j in range(2):
        total_atoms = len(atoms)
        CnearH = near(total_atoms-3, atoms, 'H', limit=False)
        del atoms[CnearH]
        i1 = numjudge(i1, CnearH)
        i2 = numjudge(i2, CnearH)
        Hnear1 = numjudge(Hnear1, CnearH)
        Hnear2 = numjudge(Hnear2, CnearH)
    total_atoms = len(atoms)
    mask = [0]*(total_atoms-2) + [1]*2
    if side < 2:
        atoms.set_distance(total_atoms-2, total_atoms-3, 0, fix=1, mask=mask)
        atoms.set_distance(total_atoms-2, total_atoms-3, 1.0900, fix=1, mask=mask)
        while(1):
            if atoms.get_distance(total_atoms-2, Hnear1) < 1.3:
                atoms.set_distance(total_atoms-2, Hnear1, 1.8, fix=1, mask=mask)
            if atoms.get_distance(total_atoms-2, Hnear2) < 1.3:
                atoms.set_distance(total_atoms-2, Hnear2, 1.8, fix=1, mask=mask)
            xoy = near(total_atoms-1, atoms, limit=False)
            #if atoms.get_distance(xoy, total_atoms-2) <= 0.85:
            #    atoms.set_distance(xoy, total_atoms-2, 1, mask=mask)
            atoms.set_angle(Hnear1, total_atoms-3, total_atoms-2, 111.0, mask=mask)
            atoms.set_angle(Hnear2, total_atoms-3, total_atoms-2, 111.0, mask=mask)
            atoms.set_distance(total_atoms-2, total_atoms-3, 1.0900, fix=1, mask=mask)
            if (atoms.get_distance(total_atoms-2, Hnear1) > 1.4 and 
                atoms.get_distance(total_atoms-2, Hnear2)) > 1.4:
                if (abs(atoms.get_angle(Hnear1, total_atoms-3, total_atoms-2)
                        - atoms.get_angle(Hnear2, total_atoms-3, total_atoms-2)) < 1):
                    #print(atoms.get_angle(Hnear1, total_atoms-3, total_atoms-2)
                    #.     - atoms.get_angle(Hnear2, total_atoms-3, total_atoms-2))
                    break
        #atoms.set_dihedral(total_atoms-2, total_atoms-3, Hnear1, i1, 60, mask=mask)
        mask[-2] = 0
        #atoms.set_distance(total_atoms-1, total_atoms-3, 0, fix=1, mask=mask)
        #atoms.set_distance(total_atoms-1, total_atoms-3, 1.0900, fix=1, mask=mask)
        #while(1):
        #    if atoms.get_distance(total_atoms-1, Hnear1) < 1.3:
        #        atoms.set_distance(total_atoms-1, Hnear1, 1.8, fix=1, mask=mask)
        #    if atoms.get_distance(total_atoms-1, Hnear2) < 1.3:
        #        atoms.set_distance(total_atoms-1, Hnear2, 1.8, fix=1, mask=mask)
        #    atoms.set_angle(Hnear1, total_atoms-3, total_atoms-1, 111.0, mask=mask)
        #    atoms.set_angle(Hnear2, total_atoms-3, total_atoms-1, 111.0, mask=mask)
        #    atoms.set_distance(total_atoms-1, total_atoms-3, 1.0900, fix=1, mask=mask)
        #    if atoms.get_distance(total_atoms-1, Hnear1) > 1.4 and
        #       atoms.get_distance(total_atoms-1, Hnear2) > 1.4:
        #        if abs(atoms.get_angle(Hnear1, total_atoms-3, total_atoms-1)
        #               - atoms.get_angle(Hnear2, total_atoms-3, total_atoms-1)) < 1:
        #            print(atoms.get_angle(Hnear1, total_atoms-3, total_atoms-1)
        #                  - atoms.get_angle(Hnear2, total_atoms-3, total_atoms-1))
        #            break
        #atoms.set_dihedral(total_atoms-1, total_atoms-3, Hnear1, i1, 170, mask=mask)
        #view(atoms)
        #mask[-2] = 0
        #mask[-1] = 1
        atoms.set_distance(total_atoms-1, total_atoms-2, 0, fix=1, mask=mask)
        atoms.set_dihedral(total_atoms-1, total_atoms-3, Hnear1, i1,\
                           atoms.get_dihedral(total_atoms-1, total_atoms-3, Hnear1, i1)+110, mask=mask)
        xoy = near(total_atoms-1, atoms, limit=False)
        if atoms.get_distance(xoy, total_atoms-1) <= 0.9:
            atoms.set_dihedral(total_atoms-1, total_atoms-3, Hnear1,\
                               i1, atoms.get_dihedral(total_atoms-1, total_atoms-3, Hnear1, i1)-220, mask=mask)
    else:
        atoms.set_distance(total_atoms-2, total_atoms-3, 0, fix=1, mask=mask)
        atoms.set_distance(total_atoms-2, total_atoms-3, 1.0900, fix=1, mask=mask)
        while(1):
            if atoms.get_distance(total_atoms-2, Hnear1) < 1.3:
                atoms.set_distance(total_atoms-2, Hnear1, 1.8, fix=1, mask=mask)
            if atoms.get_distance(total_atoms-2, Hnear2) < 1.3:
                atoms.set_distance(total_atoms-2, Hnear2, 1.8, fix=1, mask=mask)
            atoms.set_angle(Hnear1, total_atoms-3, total_atoms-2, 111.0, mask=mask)
            atoms.set_angle(Hnear2, total_atoms-3, total_atoms-2, 111.0, mask=mask)
            atoms.set_distance(total_atoms-2, total_atoms-3, 1.0900, fix=1, mask=mask)
            if (atoms.get_distance(total_atoms-2, Hnear1) > 1.4 and 
                atoms.get_distance(total_atoms-2, Hnear2) > 1.4):
                break
        mask[-2] = 0
        atoms.set_distance(total_atoms-1, total_atoms-2, 0, fix=1, mask=mask)
        #atoms.set_distance(total_atoms-1, total_atoms-3, 1.0900, fix=1, mask=mask)
        #while(1):
        #    if atoms.get_distance(total_atoms-1, Hnear1)<1.3:
        #        atoms.set_distance(total_atoms-1, Hnear1, 1.8, fix=1, mask=mask)
        #    if atoms.get_distance(total_atoms-1, Hnear2)<1.3:
        #        atoms.set_distance(total_atoms-1, Hnear2, 1.8, fix=1, mask=mask)
        #    atoms.set_angle(Hnear1, total_atoms-3, total_atoms-1, 111.0, mask=mask)
        #    atoms.set_angle(Hnear2, total_atoms-3, total_atoms-1, 111.0, mask=mask)
        #    atoms.set_distance(total_atoms-1, total_atoms-3, 1.0900, fix=1, mask=mask)
        #    if atoms.get_distance(total_atoms-1, 4)>1.4 and atoms.get_distance(total_atoms-1, 6)>1.4:
        #        break
        atoms.set_dihedral(total_atoms-1, total_atoms-3, Hnear1, i1,\
                           atoms.get_dihedral(total_atoms-1, total_atoms-3, Hnear1, i1)+110, mask=mask)
        xoy = near(total_atoms-1, atoms, limit=False)
        if atoms.get_distance(xoy, total_atoms-1) <= 0.9:
            atoms.set_dihedral(total_atoms-1, total_atoms-3, Hnear1,\
                               i1, atoms.get_dihedral(total_atoms-1, total_atoms-3, Hnear1, i1)-220, mask=mask)
    needelements = []
    places = []
    for j in range(3):
        try:
            needelements.append(strict[j+2])
            places.append(j+2)
        except:
            pass
    for i in range(len(needelements)):
        total_atoms = len(atoms)
        mask = [0]*total_atoms
        if places[i] == 2:
            mask[Hnear1] = 1
            elements = atoms.get_chemical_symbols()
            elements[Hnear1] = needelements[i]
            atoms.set_chemical_symbols(elements)
            if needelements[i] == 'N':
                for k in range(10):
                    if state[0] == 1:
                        atoms.set_distance(Hnear1, i1, 1.4480, fix=1, mask=mask)
                    elif state[0] == 2 or state[0] == 6:
                        atoms.set_distance(Hnear1, i1, 1.3400, fix=1, mask=mask)
                    atoms.set_distance(Hnear1, total_atoms-3, 1.4480, fix=1, mask=mask)
                mask[Hnear1] = 0
                total_atoms = len(atoms)
                CnearH = near(Hnear1, atoms, 'H', limit=False)
                del atoms[CnearH]
                Hnear1 = numjudge(Hnear1, CnearH)
                i1 = numjudge(i1, CnearH)
                i2 = numjudge(i2, CnearH)
                CnearH = near(Hnear1, atoms, 'H', limit=False)
                mask[CnearH] = 1
                for k in range(10):
                    atoms.set_angle(i1, Hnear1, CnearH, 109.5, mask=mask)
                    atoms.set_angle(total_atoms-3, Hnear1, CnearH, 109.5, mask=mask)
                atoms.set_dihedral(i2, i1, Hnear1, CnearH, -150, mask=mask)
                atoms.set_distance(CnearH, Hnear1, 1.0100, fix=1)
            elif needelements[i] == 'O':
                for k in range(10):
                    if state[0] == 1:
                        atoms.set_distance(Hnear1, i1, 1.4100, fix=1, mask=mask)
                    elif state[0] == 2 or state[0] == 6:
                        atoms.set_distance(Hnear1, i1, 1.3640, fix=1, mask=mask)
                    atoms.set_distance(Hnear1, total_atoms-3, 1.4100, fix=1, mask=mask)
                for j in range(2):
                    total_atoms = len(atoms)
                    CnearH = near(Hnear1, atoms, 'H', limit=False)
                    del atoms[CnearH]
                    Hnear1 = numjudge(Hnear1, CnearH)
        if places[i] == 3:
            mask[total_atoms-3] = 1
            elements = atoms.get_chemical_symbols()
            elements[total_atoms-3] = needelements[i]
            atoms.set_chemical_symbols(elements)
            if needelements[i] == 'N':
                for k in range(10):
                    atoms.set_distance(total_atoms-3, Hnear1, 1.4480, fix=1, mask=mask)
                    atoms.set_distance(total_atoms-3, Hnear2, 1.4480, fix=1, mask=mask)
                mask[total_atoms-3] = 0
                total_atoms = len(atoms)
                CnearH = near(total_atoms-3, atoms, 'H', limit=False)
                del atoms[CnearH]
                Hnear1 = numjudge(Hnear1, CnearH)
                i1 = numjudge(i1, CnearH)
                i2 = numjudge(i2, CnearH)
                CnearH = near(total_atoms-3, atoms, 'H', limit=False)
                mask[CnearH] = 1
                for k in range(10):
                    atoms.set_angle(Hnear1, total_atoms-3, CnearH, 109.5, mask=mask)
                    atoms.set_angle(Hnear2, total_atoms-3, CnearH, 109.5, mask=mask)
                atoms.set_dihedral(i1, Hnear1, total_atoms-3, CnearH, -150, mask=mask)
                atoms.set_distance(CnearH, total_atoms-3, 1.0100, fix=1)
            elif needelements[i] == 'O':
                for k in range(10):
                    atoms.set_distance(total_atoms-3, Hnear1, 1.4100, fix=1, mask=mask)
                    atoms.set_distance(total_atoms-3, Hnear2, 1.4100, fix=1, mask=mask)
                for j in range(2):
                    total_atoms = len(atoms)
                    CnearH = near(total_atoms-3+j, atoms, 'H', limit=False)
                    del atoms[CnearH]
        if places[i] == 4:
            mask[Hnear2] = 1
            elements = atoms.get_chemical_symbols()
            elements[Hnear2] = needelements[i]
            atoms.set_chemical_symbols(elements)
            if needelements[i] == 'N':
                for k in range(10):
                    if state[0] == 1:
                        atoms.set_distance(Hnear2, i2, 1.4480, fix=1, mask=mask)
                    elif state[0] == 2 or state[0] == 6:
                        atoms.set_distance(Hnear2, i2, 1.3400, fix=1, mask=mask)
                    atoms.set_distance(Hnear2, total_atoms-3, 1.4480, fix=1, mask=mask)
                mask[Hnear2] = 0
                total_atoms = len(atoms)
                CnearH = near(Hnear2, atoms, 'H', limit=False)
                del atoms[CnearH]
                Hnear1 = numjudge(Hnear1, CnearH)
                i1 = numjudge(i1, CnearH)
                i2 = numjudge(i2, CnearH)
                CnearH = near(Hnear2, atoms, 'H', limit=False)
                mask[CnearH] = 1
                for k in range(10):
                    atoms.set_angle(i2, Hnear2, CnearH, 109.5, mask=mask)
                    atoms.set_angle(total_atoms-3, Hnear2, CnearH, 109.5, mask=mask)
                atoms.set_dihedral(i1, i2, Hnear2, CnearH, -150, mask=mask)
                atoms.set_distance(CnearH, Hnear2, 1.0100, fix=1)
            elif needelements[i] == 'O':
                for k in range(10):
                    if state[0] == 1:
                        atoms.set_distance(Hnear2, i2, 1.4100, fix=1, mask=mask)
                    elif state[0] == 2 or state[0] == 6:
                        atoms.set_distance(Hnear2, i2, 1.3640, fix=1, mask=mask)
                    atoms.set_distance(Hnear2, total_atoms-3, 1.4100, fix=1, mask=mask)
                for j in range(2):
                    total_atoms = len(atoms)
                    CnearH = near(Hnear2, atoms, 'H', limit=False)
                    del atoms[CnearH]
                    Hnear2 = numjudge(Hnear2, CnearH)
    return atoms

def bind6(atoms, bond, state=[1, 1], strict={1:'X'}, side=0, shuffle=0, less5=False):
    i1 = bond[0]
    i2 = bond[1]
    if side == 0:
        Hnear1 = near(i1, atoms, 'H')
        Hnear2 = near(i2, atoms, 'H')
    elif side == 1:
        Hnear1 = near(i1, atoms, 'H', reverse=True)
        Hnear2 = near(i2, atoms, 'H', reverse=True)
    elif side == 2:
        Hnear1 = near(i1, atoms, 'H')
        Hnear2 = near(i2, atoms, 'H', reverse=True)
    elif side == 3:
        Hnear1 = near(i1, atoms, 'H', reverse=True)
        Hnear2 = near(i2, atoms, 'H')
    print(Hnear1, Hnear2)
    if less5 == True and side >= 2:
        total_atoms = len(atoms)
        mask = [0]*total_atoms
        mask[Hnear1] = 1
        mask[Hnear2] = 1
        atoms.set_angle(i2, i1, Hnear1, 107.8, mask=mask)
        mask[Hnear1] = 0
        atoms.set_angle(i1, i2, Hnear2, 107.8, mask=mask)
        atoms.set_dihedral(Hnear1, i1, i2, Hnear2, 55, mask=mask)
        xoy = near(Hnear2, atoms, limit=False)
        if atoms.get_distance(xoy, Hnear2) <= 0.9:
            atoms.set_dihedral(Hnear1, i1, i2, Hnear2, -55, mask=mask)
        xoy = near(i2, atoms, target='Not H')
        if xoy == i1:
            xoy = near(i2, atoms, target='Not H', reverse=True)
        if atoms.get_angle(xoy, i2, Hnear2) <= 90:
            atoms.set_dihedral(Hnear1, i1, i2, Hnear2, -55, mask=mask)
    if shuffle == 0:
        atoms = CH3(atoms, Hnear1, state[0])
        atoms = CH3(atoms, Hnear2, state[1])
    elif shuffle == 1:
        atoms = CH3(atoms, Hnear1, state[0], ring=True)
        atoms = CH3(atoms, Hnear2, state[1], ring=True)
    elif shuffle == 2:
        atoms = CH3(atoms, Hnear1, state[0], ring=True)
        atoms = CH3(atoms, Hnear2, state[1])
    elif shuffle == 3:
        atoms = CH3(atoms, Hnear1, state[0])
        atoms = CH3(atoms, Hnear2, state[1], ring=True)
    atoms.append('C')
    atoms.append('C')
    for j in range(4):
        atoms.append('H')
    x = atoms.get_positions()
    x[-5][0] = x[-5][0] + 50
    x[-5][1] = x[-5][1] + 50
    x[-5][2] = x[-5][2] + 50
    x[-6][0] = x[-6][0] - 50
    x[-6][1] = x[-6][1] - 50
    x[-6][2] = x[-6][2] - 50
    for i in range(-2, 0):
        x[2*i][0] = x[2*i][0] - 50
        x[2*i][1] = x[2*i][1] - 50
        x[2*i][2] = x[2*i][2] - 50
        x[2*i+1][0] = x[2*i+1][0] + 50
        x[2*i+1][1] = x[2*i+1][1] + 50
        x[2*i+1][2] = x[2*i+1][2] + 50
    atoms.set_positions(x)
    total_atoms = len(atoms)
    mask = [0]*(total_atoms-6) + [1]*6
    atoms.set_angle(i1, Hnear1, total_atoms-6, 107.8, mask=mask)
    if state[0] == 1:
        atoms.set_distance(total_atoms-6, Hnear1, 1.5290, fix=1, mask=mask)
    elif state[0] == 2:
        atoms.set_distance(total_atoms-6, Hnear1, 1.5095, fix=1, mask=mask)
    elif state[0] == 6:
        atoms.set_distance(total_atoms-6, Hnear1, 1.5105, fix=1, mask=mask)
    if less5 == False:
        atoms.set_dihedral(i2, i1, Hnear1, total_atoms-6, 47.9, mask=mask)
        if atoms.get_distance(total_atoms-6, Hnear2) >= 2.7:
            atoms.set_dihedral(i2, i1, Hnear1, total_atoms-6, -47.9, mask=mask)
    else:
        atoms.set_dihedral(i2, i1, Hnear1, total_atoms-6, -47.9, mask=mask)
        if atoms.get_distance(total_atoms-6, Hnear2) >= 2.7:
            atoms.set_dihedral(i2, i1, Hnear1, total_atoms-6, 47.9, mask=mask)
            
    mask[-6] = 0
    atoms.set_angle(i2, Hnear2, total_atoms-5, 107.8, mask=mask)
    for k in range(10):
        #print(atoms.get_dihedral(i1, i2, Hnear2, total_atoms-5))
        if (less5 == True and atoms.get_dihedral(Hnear2, i2, i1, Hnear1) > 10 and
            atoms.get_dihedral(Hnear2, i2, i1, Hnear1) < 350):
            if (atoms.get_dihedral(i1, i2, Hnear2, total_atoms-5) <= 30
                and atoms.get_dihedral(i1, i2, Hnear2, total_atoms-5) >= 0):
                atoms.set_dihedral(i1, i2, Hnear2, total_atoms-5, 47.9, mask=mask)
            if (atoms.get_dihedral(i1, i2, Hnear2, total_atoms-5) >= 330
                and atoms.get_dihedral(i1, i2, Hnear2, total_atoms-5) <= 360):
                atoms.set_dihedral(i1, i2, Hnear2, total_atoms-5, -47.9, mask=mask)
        else:
            if (atoms.get_dihedral(i1, i2, Hnear2, total_atoms-5) <= 30 and
                atoms.get_dihedral(i1, i2, Hnear2, total_atoms-5) >= 0):
                atoms.set_dihedral(i1, i2, Hnear2, total_atoms-5, -47.9, mask=mask)
            if (atoms.get_dihedral(i1, i2, Hnear2, total_atoms-5) >= 330 and
                atoms.get_dihedral(i1, i2, Hnear2, total_atoms-5) <= 360):
                atoms.set_dihedral(i1, i2, Hnear2, total_atoms-5, 47.9, mask=mask)
        atoms.set_distance(total_atoms-5, total_atoms-6, 1.5290, fix=1, mask=mask)
        if state[1] == 1:
            atoms.set_distance(total_atoms-5, Hnear2, 1.5290, fix=1, mask=mask)
        elif state[1] == 2:
            atoms.set_distance(total_atoms-5, Hnear2, 1.5095, fix=1, mask=mask)
        elif state[1] == 6:
            atoms.set_distance(total_atoms-5, Hnear2, 1.5105, fix=1, mask=mask)
    mask[-5] = 0
    for j in range(2):
        total_atoms = len(atoms)
        CnearH = near(total_atoms-6+j, atoms, 'H', limit=False)
        del atoms[CnearH]
        i1 = numjudge(i1, CnearH)
        i2 = numjudge(i2, CnearH)
        Hnear1 = numjudge(Hnear1, CnearH)
        Hnear2 = numjudge(Hnear2, CnearH)
    total_atoms = len(atoms)
    mask = [0]*(total_atoms-4)+[1]*4
    atoms.set_distance(total_atoms-4, total_atoms-6, 1.0900, fix=1, mask=mask)
    atoms.set_dihedral(total_atoms-4, total_atoms-6, Hnear1, i1, -110, mask=mask)
    for k in range(10):
        atoms.set_angle(Hnear1, total_atoms-6, total_atoms-4, 111.5, mask=mask)
        atoms.set_angle(total_atoms-5, total_atoms-6,\
                        total_atoms-4, 111.5, mask=mask)
    mask[-4] = 0
    atoms.set_distance(total_atoms-3, total_atoms-4, 0, fix=1, mask=mask)
    atoms.set_dihedral(total_atoms-3, total_atoms-6, Hnear1, i1,\
                       atoms.get_dihedral(total_atoms-3,\
                                          total_atoms-6, Hnear1, i1)-110, mask=mask)
    xoy = near(total_atoms-3, atoms, limit=False)
    if atoms.get_distance(xoy, total_atoms-3) <= 0.9:
        atoms.set_dihedral(total_atoms-3, total_atoms-6, Hnear1, i1,\
                           atoms.get_dihedral(total_atoms-3,\
                                              total_atoms-6, Hnear1, i1)+220, mask=mask)
    mask[-3] = 0
    atoms.set_distance(total_atoms-2, total_atoms-5, 0, fix=1, mask=mask)
    atoms.set_distance(total_atoms-2, total_atoms-5, 1.0900, fix=1, mask=mask)
    atoms.set_dihedral(total_atoms-2, total_atoms-5, Hnear2, i2, 110, mask=mask)
    for k in range(10):
        atoms.set_angle(Hnear2, total_atoms-5, total_atoms-2, 111.5, mask=mask)
        atoms.set_angle(total_atoms-6, total_atoms-5,\
                        total_atoms-2, 111.5, mask=mask)
    mask[-2] = 0
    atoms.set_distance(total_atoms-1, total_atoms-2, 0, fix=1, mask=mask)
    atoms.set_dihedral(total_atoms-1, total_atoms-5, Hnear2, i2,\
                       atoms.get_dihedral(total_atoms-1,\
                                          total_atoms-5, Hnear2, i2)-110, mask=mask)
    xoy = near(total_atoms-1, atoms, limit=False)
    if atoms.get_distance(xoy, total_atoms-1) <= 0.9:
        atoms.set_dihedral(total_atoms-1, total_atoms-5, Hnear2, i2,\
                           atoms.get_dihedral(total_atoms-1,\
                                              total_atoms-5, Hnear2, i2)+220, mask=mask)

#    for k in range(10):
#        atoms.set_angle(Hnear2, total_atoms-5, total_atoms-1, 111.5, mask=mask)
#        atoms.set_angle(total_atoms-6, total_atoms-5, total_atoms-1, 111.5, mask=mask)
        
    mask[-1] = 0
    needelements = []
    places = []
    for j in range(4):
        try:
            needelements.append(strict[j+2])
            places.append(j+2)
        except:
            pass
    error = 0
    for i in range(len(needelements)):
        total_atoms = len(atoms)
        mask = [0]*total_atoms
        if places[i] == 2:
            mask[Hnear1] = 1
            elements = atoms.get_chemical_symbols()
            elements[Hnear1] = needelements[i]
            atoms.set_chemical_symbols(elements)
            if needelements[i] == 'N':
                for k in range(10):
                    if state[0] == 1:
                        atoms.set_distance(Hnear1, i1, 1.4480, fix=1, mask=mask)
                    elif state[0] == 2 or state[0] == 6:
                        atoms.set_distance(Hnear1, i1, 1.3400, fix=1, mask=mask)
                    atoms.set_distance(Hnear1, total_atoms-6, 1.4480, fix=1, mask=mask)
                mask[Hnear1] = 0
                total_atoms = len(atoms)
                CnearH = near(Hnear1, atoms, 'H', limit=False)
                del atoms[CnearH]
                Hnear1 = numjudge(Hnear1, CnearH)
                i1 = numjudge(i1, CnearH)
                i2 = numjudge(i2, CnearH)
                CnearH = near(Hnear1, atoms, 'H', limit=False)
                mask[CnearH] = 1
                for k in range(10):
                    atoms.set_angle(i1, Hnear1, CnearH, 109.5, mask=mask)
                    atoms.set_angle(total_atoms-6, Hnear1, CnearH, 109.5, mask=mask)
                atoms.set_dihedral(i2, i1, Hnear1, CnearH, -150, mask=mask)
                atoms.set_distance(CnearH, Hnear1, 1.0100, fix=1)
            elif needelements[i] == 'O':
                for k in range(10):
                    if state[0] == 1:
                        atoms.set_distance(Hnear1, i1, 1.4100, fix=1, mask=mask)
                    elif state[0] == 2 or state[0] == 6:
                        atoms.set_distance(Hnear1, i1, 1.3640, fix=1, mask=mask)
                    atoms.set_distance(Hnear1, total_atoms-6, 1.4100, fix=1, mask=mask)
                for j in range(2):
                    total_atoms = len(atoms)
                    CnearH = near(Hnear1, atoms, 'H', limit=False)
                    del atoms[CnearH]
                    Hnear1 = numjudge(Hnear1, CnearH)
        if places[i] == 3:
            mask[total_atoms-6+error] = 1
            elements = atoms.get_chemical_symbols()
            elements[total_atoms-6+error] = needelements[i]
            atoms.set_chemical_symbols(elements)
            if needelements[i] == 'N':
                for k in range(10):
                    atoms.set_distance(total_atoms-6+error, Hnear1, 1.4480, fix=1, mask=mask)
                    atoms.set_distance(total_atoms-6+error, total_atoms-5+error,\
                                       1.4480, fix=1, mask=mask)
                mask[total_atoms-6+error] = 0
                total_atoms = len(atoms)
                CnearH = near(total_atoms-6+error, atoms, 'H', limit=False)
                del atoms[CnearH]
                Hnear1 = numjudge(Hnear1, CnearH)
                i1 = numjudge(i1, CnearH)
                i2 = numjudge(i2, CnearH)
                CnearH = near(total_atoms-6+error, atoms, 'H', limit=False)
                mask[CnearH] = 1
                atoms.set_dihedral(i1, Hnear1, total_atoms-6+error, CnearH, -150, mask=mask)
                for k in range(10):
                    atoms.set_angle(Hnear1, total_atoms-6+error, CnearH, 109.5, mask=mask)
                    atoms.set_angle(total_atoms-5+error, total_atoms-6+error, CnearH,\
                                    109.5, mask=mask)
                atoms.set_distance(CnearH, total_atoms-6+error, 1.0100, fix=1)
                error = error+1
            elif needelements[i] == 'O':
                for k in range(10):
                    atoms.set_distance(total_atoms-6+error, Hnear1, 1.4100, fix=1, mask=mask)
                    atoms.set_distance(total_atoms-6+error, total_atoms-5-error, 1.4100,\
                                       fix=1, mask=mask)
                for j in range(2):
                    total_atoms = len(atoms)
                    CnearH = near(total_atoms-6+error+j, atoms, 'H', limit=False)
                    del atoms[CnearH]
                error = error+2
        if places[i] == 4:
            mask[total_atoms-5+error] = 1
            elements = atoms.get_chemical_symbols()
            elements[total_atoms-5-error] = needelements[i]
            atoms.set_chemical_symbols(elements)
            if needelements[i] == 'N':
                for k in range(10):
                    atoms.set_distance(total_atoms-5+error, total_atoms-6+error,\
                                       1.4480, fix=1, mask=mask)
                    atoms.set_distance(total_atoms-5+error, Hnear2, 1.4480, fix=1, mask=mask)
                mask[total_atoms-5+error] = 0
                total_atoms = len(atoms)
                CnearH = near(total_atoms-5+error, atoms, 'H', limit=False)
                del atoms[CnearH]
                Hnear1 = numjudge(Hnear1, CnearH)
                i1 = numjudge(i1, CnearH)
                i2 = numjudge(i2, CnearH)
                CnearH = near(total_atoms-5+error, atoms, 'H', limit=False)
                mask[CnearH] = 1
                atoms.set_dihedral(i2, Hnear2, total_atoms-5+error, CnearH, -150, mask=mask)
                for k in range(10):
                    atoms.set_angle(Hnear2, total_atoms-5+error, CnearH, 109.5, mask=mask)
                    atoms.set_angle(total_atoms-6+error, total_atoms-5+error,\
                                    CnearH, 109.5, mask=mask)
                atoms.set_distance(CnearH, total_atoms-5+error, 1.0100, fix=1)
                error = error+1
            elif needelements[i] == 'O':
                for k in range(10):
                    atoms.set_distance(total_atoms-5+error, total_atoms-6+error,\
                                       1.4100, fix=1, mask=mask)
                    atoms.set_distance(total_atoms-5+error, Hnear2, 1.4100, fix=1, mask=mask)
                for j in range(2):
                    total_atoms = len(atoms)
                    CnearH = near(total_atoms-5-error+j, atoms, 'H', limit=False)
                    del atoms[CnearH]
                error = error+2
        if places[i] == 5:
            mask[Hnear2] = 1
            elements = atoms.get_chemical_symbols()
            elements[Hnear2] = needelements[i]
            atoms.set_chemical_symbols(elements)
            if needelements[i] == 'N':
                for k in range(10):
                    if state[0] == 1:
                        atoms.set_distance(Hnear2, i2, 1.4480, fix=1, mask=mask)
                    elif state[0] == 2 or state[0] == 6:
                        atoms.set_distance(Hnear2, i2, 1.3400, fix=1, mask=mask)
                    atoms.set_distance(Hnear2, total_atoms-5+error, 1.4480, fix=1, mask=mask)
                mask[Hnear2] = 0
                total_atoms = len(atoms)
                CnearH = near(Hnear2, atoms, 'H', limit=False)
                del atoms[CnearH]
                Hnear1 = numjudge(Hnear1, CnearH)
                i1 = numjudge(i1, CnearH)
                i2 = numjudge(i2, CnearH)
                CnearH = near(Hnear2, atoms, 'H', limit=False)
                mask[CnearH] = 1
                for k in range(10):
                    atoms.set_angle(i2, Hnear2, CnearH, 109.5, mask=mask)
                    atoms.set_angle(total_atoms-5+error, Hnear2, CnearH, 109.5, mask=mask)
                atoms.set_dihedral(i1, i2, Hnear2, CnearH, -150, mask=mask)
                atoms.set_distance(CnearH, Hnear2, 1.0100, fix=1)
            elif needelements[i] == 'O':
                for k in range(10):
                    if state[0] == 1:
                        atoms.set_distance(Hnear2, i2, 1.4100, fix=1, mask=mask)
                    elif state[0] == 2 or state[0] == 6:
                        atoms.set_distance(Hnear2, i2, 1.3640, fix=1, mask=mask)
                    atoms.set_distance(Hnear2, total_atoms-5+error, 1.4100, fix=1, mask=mask)
                for j in range(2):
                    total_atoms = len(atoms)
                    CnearH = near(Hnear2, atoms, 'H', limit=False)
                    del atoms[CnearH]
                    Hnear2 = numjudge(Hnear2, CnearH)
    return atoms

def bindconj5(atoms, bond, state=[1, 1], strict={1:'X'}):
    try:
        if strict[1] == 'X':
            strict[3] = 'N'
    except:
        pass
    i1 = bond[0]
    i2 = bond[1]
    Cnear1 = near(i1, atoms, 'Not H')
    Cnear2 = near(i2, atoms, 'Not H', reverse=True)
    total_atoms = len(atoms)
    mask = [0] * total_atoms
    mask[i1] = 1
    mask[i2] = 1
    for j in range(3):
        try:
            element = strict[j+2]
            place = j + 2
        except:
            pass
    bondjudge5([Cnear1, Cnear2], atoms, [i1, i2], state, mask, place)
    for j in range(2):
        CnearH = near(i1, atoms, 'H')
        del atoms[CnearH]
        i1 = numjudge(i1, CnearH)
        i2 = numjudge(i2, CnearH)
    for j in range(2):
        CnearH = near(i2, atoms, 'H')
        del atoms[CnearH]
        i1 = numjudge(i1, CnearH)
        i2 = numjudge(i2, CnearH)
    if place == 2:
        atoms.append(element)
        atoms.append('C')
        atoms.append('C')
    elif place == 3:
        atoms.append('C')
        atoms.append('C')
        atoms.append(element)
    elif place == 4:
        atoms.append('C')
        atoms.append(element)
        atoms.append('C')
    for j in range(3):
        atoms.append('H')
    x = atoms.get_positions()
    x[-6][0] = x[-6][0] - 100
    x[-6][1] = x[-6][1] - 50
    x[-6][2] = x[-6][2] - 50
    x[-5][0] = x[-5][0] + 50
    x[-5][1] = x[-5][1] + 50
    x[-5][2] = x[-5][2] + 50
    for j in range(-4, 0):
        x[j][0] = x[j][0] + 50
        x[j][1] = x[j][1] + 50
        x[j][2] = x[j][2] + 50
    atoms.set_positions(x)
    total_atoms = len(atoms)
    mask = [0]*(total_atoms-6)+[1]*6
    if place == 2:
        if element == 'N':
            atoms.set_angle(i2, i1, total_atoms-6, 107.7, mask=mask)
            if state[0] == 1:
                atoms.set_distance(total_atoms-6, i1, 1.3810, fix=1, mask=mask)
            elif state[0] == 2:
                atoms.set_distance(total_atoms-6, i1, 1.3740, fix=1, mask=mask)
        elif element == 'O':
            atoms.set_angle(i2, i1, total_atoms-6, 110.6, mask=mask)
            atoms.set_distance(total_atoms-6, i1, 1.3600, fix=1, mask=mask)
        mask[-6] = 0
        atoms.set_angle(i1, i2, total_atoms-5, 107.3, mask=mask)
        atoms.set_distance(total_atoms-5, i2, 1.4240, fix=1, mask=mask)
        atoms.set_dihedral(total_atoms-6, i1, i2, total_atoms-5, 0.5, mask=mask)
        mask[-5] = 0
        if element == 'N':
            for k in range(10):
                atoms.set_distance(total_atoms-4, total_atoms-6, 1.3810, fix=1, mask=mask)
                atoms.set_distance(total_atoms-4, total_atoms-5, 1.3670, fix=1, mask=mask)
            mask[-4] = 0
            #atoms.set_distance(total_atoms-1, total_atoms-6, 0, fix=1, mask=mask)
            atoms.set_dihedral(total_atoms-5, total_atoms-4, total_atoms-6,\
                               total_atoms-1, 0.5, mask=mask)
            for k in range(10):
                atoms.set_angle(total_atoms-4, total_atoms-6, total_atoms-1, 124.5, mask=mask)
                atoms.set_angle(i1, total_atoms-6, total_atoms-1, 124.5, mask=mask)
            atoms.set_distance(total_atoms-1, total_atoms-6, 1.0100, fix=1, mask=mask)
            mask[-1] = 0
        elif element == 'O':
            for k in range(10):
                atoms.set_distance(total_atoms-4, total_atoms-6, 1.3600, fix=1, mask=mask)
                atoms.set_distance(total_atoms-4, total_atoms-5, 1.3670, fix=1, mask=mask)
            mask[-4] = 0
        atoms.set_distance(total_atoms-3, total_atoms-4, 1.0100, fix=1, mask=mask)
        atoms.set_dihedral(total_atoms-3, total_atoms-4, total_atoms-5,\
                           i2, 0.5, mask=mask)
        for k in range(10):
            atoms.set_angle(total_atoms-6, total_atoms-4, total_atoms-3, 126.5, mask=mask)
            atoms.set_angle(total_atoms-5, total_atoms-4, total_atoms-3, 126.5, mask=mask)
        mask[-3] = 0
        atoms.set_distance(total_atoms-2, total_atoms-5, 1.0100, fix=1, mask=mask)
        atoms.set_dihedral(total_atoms-2, total_atoms-5, i2, i1, 0.5, mask=mask)
        for k in range(10):
            atoms.set_angle(i2, total_atoms-5, total_atoms-2, 126.5, mask=mask)
            atoms.set_angle(total_atoms-4, total_atoms-5, total_atoms-2, 126.5, mask=mask)
        mask[-2] = 0
    elif place == 3:
        atoms.set_angle(i2, i1, total_atoms-6, 107.3, mask=mask)
        atoms.set_distance(total_atoms-6, i1, 1.3670, fix=1, mask=mask)
        mask[-6] = 0
        atoms.set_angle(i1, i2, total_atoms-5, 107.3, mask=mask)
        atoms.set_distance(total_atoms-5, i2, 1.3670, fix=1, mask=mask)
        atoms.set_dihedral(total_atoms-6, i1, i2, total_atoms-5, 0.5, mask=mask)
        mask[-5] = 0
        if element == 'N':
            for k in range(10):
                atoms.set_distance(total_atoms-4, total_atoms-6, 1.3810, fix=1, mask=mask)
                atoms.set_distance(total_atoms-4, total_atoms-5, 1.3810, fix=1, mask=mask)
            mask[-4] = 0
            atoms.set_distance(total_atoms-1, total_atoms-4, 0, fix=1, mask=mask)
            atoms.set_distance(total_atoms-1, total_atoms-4, 1.0100, fix=1, mask=mask)
            atoms.set_dihedral(i1, total_atoms-6, total_atoms-4,\
                               total_atoms-1, 110, mask=mask)
            for k in range(10):
                atoms.set_angle(total_atoms-6, total_atoms-4, total_atoms-1, 124.5, mask=mask)
                atoms.set_angle(total_atoms-5, total_atoms-4, total_atoms-1, 124.5, mask=mask)
            mask[-1] = 0
        elif element == 'O':
            for k in range(10):
                atoms.set_distance(total_atoms-4, total_atoms-6, 1.3600, fix=1, mask=mask)
                atoms.set_distance(total_atoms-4, total_atoms-5, 1.3600, fix=1, mask=mask)
            mask[-4] = 0
        atoms.set_distance(total_atoms-3, total_atoms-6, 1.0100, fix=1, mask=mask)
        atoms.set_dihedral(total_atoms-3, total_atoms-6, i1, i2, 0.5, mask=mask)
        for k in range(10):
            atoms.set_angle(i1, total_atoms-6, total_atoms-3, 126.5, mask=mask)
            atoms.set_angle(total_atoms-4, total_atoms-6, total_atoms-3, 126.5, mask=mask)
        mask[-3] = 0
        atoms.set_distance(total_atoms-2, total_atoms-5, 1.0100, fix=1, mask=mask)
        atoms.set_dihedral(total_atoms-2, total_atoms-5, i2, i1, 0.5, mask=mask)
        for k in range(10):
            atoms.set_angle(i2, total_atoms-5, total_atoms-2, 126.5, mask=mask)
            atoms.set_angle(total_atoms-4, total_atoms-5, total_atoms-2, 126.5, mask=mask)
        mask[-2] = 0
    elif place == 4:
        atoms.set_angle(i2, i1, total_atoms-6, 107.3, mask=mask)
        atoms.set_distance(total_atoms-6, i1, 1.4240, fix=1, mask=mask)
        mask[-6] = 0
        if element == 'N':
            atoms.set_angle(i1, i2, total_atoms-5, 107.7, mask=mask)
            if state[0] == 1:
                atoms.set_distance(total_atoms-5, i2, 1.3810, fix=1, mask=mask)
            if state[0] == 2:
                atoms.set_distance(total_atoms-5, i2, 1.3740, fix=1, mask=mask)
        elif element == 'O':
            atoms.set_angle(i1, i2, total_atoms-5, 110.6, mask=mask)
            atoms.set_distance(total_atoms-5, i2, 1.3600, fix=1, mask=mask)
        atoms.set_dihedral(total_atoms-6, i1, i2, total_atoms-5, 0.5, mask=mask)
        mask[-5] = 0
        if element == 'N':
            for k in range(10):
                atoms.set_distance(total_atoms-4, total_atoms-5, 1.3810, fix=1, mask=mask)
                atoms.set_distance(total_atoms-4, total_atoms-6, 1.3670, fix=1, mask=mask)
            mask[-4] = 0
            #atoms.set_distance(total_atoms-1, total_atoms-6, 0, fix=1, mask=mask)
            atoms.set_dihedral(total_atoms-6, total_atoms-4, total_atoms-5,\
                               total_atoms-1, 0.5, mask=mask)
            for k in range(10):
                atoms.set_angle(total_atoms-4, total_atoms-5, total_atoms-1, 124.5, mask=mask)
                atoms.set_angle(i2, total_atoms-5, total_atoms-1, 124.5, mask=mask)
            atoms.set_distance(total_atoms-1, total_atoms-5, 1.0100, fix=1, mask=mask)
            mask[-1] = 0
        elif element == 'O':
            for k in range(10):
                atoms.set_distance(total_atoms-4, total_atoms-5, 1.3600, fix=1, mask=mask)
                atoms.set_distance(total_atoms-4, total_atoms-6, 1.3670, fix=1, mask=mask)
            mask[-4] = 0
        atoms.set_distance(total_atoms-3, total_atoms-4, 1.0100, fix=1, mask=mask)
        atoms.set_dihedral(total_atoms-3, total_atoms-4,\
                           total_atoms-5, i2, 0.5, mask=mask)
        for k in range(10):
            atoms.set_angle(total_atoms-6, total_atoms-4, total_atoms-3, 126.5, mask=mask)
            atoms.set_angle(total_atoms-5, total_atoms-4, total_atoms-3, 126.5, mask=mask)
        mask[-3] = 0
        atoms.set_distance(total_atoms-2, total_atoms-6, 1.0100, fix=1, mask=mask)
        atoms.set_dihedral(total_atoms-2, total_atoms-6,\
                           total_atoms-4, total_atoms-5, 0.5, mask=mask)
        for k in range(10):
            atoms.set_angle(i1, total_atoms-6, total_atoms-2, 126.5, mask=mask)
            atoms.set_angle(total_atoms-4, total_atoms-6, total_atoms-2, 126.5, mask=mask)
        mask[-2] = 0
    if element != 'N':
        del atoms[-1]
    return atoms

def bindconj6(atoms, bond, state=[1, 1], strict={1:'X'}):
    i1 = bond[0]
    i2 = bond[1]
    Cnear1 = near(i1, atoms, 'Not H')
    Cnear2 = near(i2, atoms, 'Not H', reverse=True)
    total_atoms = len(atoms)
    mask = [0]*total_atoms
    mask[i1] = 1
    mask[i2] = 1
    for j in range(4):
        try:
            element = strict[j+2]
            place = j+2
            break
        except:
            place = 1
    bondjudge6([Cnear1, Cnear2], atoms, [i1, i2], state, mask, place)
    for j in range(2):
        CnearH = near(i1, atoms, 'H')
        del atoms[CnearH]
        i1 = numjudge(i1, CnearH)
        i2 = numjudge(i2, CnearH)
    for j in range(2):
        CnearH = near(i2, atoms, 'H')
        del atoms[CnearH]
        i1 = numjudge(i1, CnearH)
        i2 = numjudge(i2, CnearH)    
    if place == 2:
        atoms.append(element)
        atoms.append('C')
        atoms.append('C')
        atoms.append('C')
    elif place == 3:
        atoms.append('C')
        atoms.append('C')
        atoms.append(element)
        atoms.append('C')
    elif place == 4:
        atoms.append('C')
        atoms.append('C')
        atoms.append('C')
        atoms.append(element)
    elif place == 5:
        atoms.append('C')
        atoms.append(element)
        atoms.append('C')
        atoms.append('C')
    else:
        for j in range(4):
            atoms.append('C')
    for j in range(4):
        atoms.append('H')
    x = atoms.get_positions()
    x[-8][0] = x[-8][0] - 100
    x[-8][1] = x[-8][1] - 50
    x[-8][2] = x[-8][2] - 50
    x[-7][0] = x[-7][0] + 50
    x[-7][1] = x[-7][1] + 50
    x[-7][2] = x[-7][2] + 50
    x[-6][0] = x[-6][0] - 50
    x[-6][1] = x[-6][1] - 50
    x[-6][2] = x[-6][2] - 50
    x[-5][0] = x[-5][0] + 50
    x[-5][1] = x[-5][1] + 50
    x[-5][2] = x[-5][2] + 50
    for j in range(-2, 0):
        x[2*j][0] = x[2*j][0] + 50
        x[2*j][1] = x[2*j][1] + 50
        x[2*j][2] = x[2*j][2] + 50
        x[2*j+1][0] = x[2*j+1][0] - 50
        x[2*j+1][1] = x[2*j+1][1] - 50
        x[2*j+1][2] = x[2*j+1][2] - 50
    atoms.set_positions(x)
    total_atoms = len(atoms)
    mask = [0]*(total_atoms-8) + [1]*8
    atoms.set_angle(i2, i1, total_atoms-8, 120, mask=mask)
    if state[0] == 2:
        atoms.set_distance(total_atoms-8, i1, 1.4040, fix=1, mask=mask)
    elif state[0] == 82:
        atoms.set_distance(total_atoms-8, i1, 1.4330, fix=1, mask=mask)
    else:
        atoms.set_distance(total_atoms-8, i1, 1.4000, fix=1, mask=mask)
    mask[-8] = 0
    atoms.set_angle(i1, i2, total_atoms-7, 120, mask=mask)
    if state[0] == 2:
        atoms.set_distance(total_atoms-7, i2, 1.4040, fix=1, mask=mask)
    elif state[0] == 82:
        atoms.set_distance(total_atoms-7, i2, 1.4330, fix=1, mask=mask)
    else:
        atoms.set_distance(total_atoms-7, i2, 1.4000, fix=1, mask=mask)
    if state[0] == 2 or state[0] == 82:
        atoms.set_dihedral(total_atoms-8, i1, i2, total_atoms-7, 5, mask=mask)
    else:
        atoms.set_dihedral(total_atoms-8, i1, i2, total_atoms-7, 0.5, mask=mask)
    mask[-7] = 0
    atoms.set_angle(i1, total_atoms-8, total_atoms-6, 120, mask=mask)
    if state[0] == 82:
        atoms.set_distance(total_atoms-6, total_atoms-8, 1.4330, fix=1, mask=mask)
    else:
        atoms.set_distance(total_atoms-6, total_atoms-8, 1.4000, fix=1, mask=mask)
    atoms.set_dihedral(total_atoms-6, total_atoms-8, i1, i2, -0.5, mask=mask)
    mask[-6] = 0
    atoms.set_angle(i2, total_atoms-7, total_atoms-5, 120, mask=mask)
    if state[0] == 82:
        atoms.set_distance(total_atoms-5, total_atoms-7, 1.4330, fix=1, mask=mask)
    else:
        atoms.set_distance(total_atoms-5, total_atoms-7, 1.4000, fix=1, mask=mask)
    atoms.set_dihedral(total_atoms-5, total_atoms-7, i2, i1, -0.5, mask=mask)
    mask[-5] = 0
    atoms.set_angle(total_atoms-4, total_atoms-8, i1, 120, mask=mask)
    atoms.set_distance(total_atoms-4, total_atoms-8, 1.0800, fix=1, mask=mask)
    atoms.set_dihedral(i2, i1, total_atoms-8, total_atoms-4, -179.5, mask=mask)
    mask[-4] = 0
    atoms.set_angle(total_atoms-3, total_atoms-7, i2, 120, mask=mask)
    atoms.set_distance(total_atoms-3, total_atoms-7, 1.0800, fix=1, mask=mask)
    atoms.set_dihedral(i1, i2, total_atoms-7, total_atoms-3, -179.5, mask=mask)
    mask[-3] = 0
    atoms.set_angle(total_atoms-8, total_atoms-6, total_atoms-2, 120, mask=mask)
    atoms.set_dihedral(total_atoms-2, total_atoms-6,\
                       total_atoms-8, i1, 179.5, mask=mask)
    atoms.set_distance(total_atoms-2, total_atoms-6, 1.0800, fix=1, mask=mask)
    mask[-2] = 0
    atoms.set_angle(total_atoms-7, total_atoms-5, total_atoms-1, 120, mask=mask)
    atoms.set_distance(total_atoms-1, total_atoms-5, 1.0800, fix=1, mask=mask)
    atoms.set_dihedral(total_atoms-8, total_atoms-6,\
                       total_atoms-7, total_atoms-1, 179.5, mask=mask)
    if place == 2:
        elements = atoms.get_chemical_symbols()
        elements[-8] = 'N'
        atoms.set_chemical_symbols(elements)
        for k in range(10):
            atoms.set_distance(total_atoms-8, total_atoms-6, 1.3390, fix=1)
            if state[0] == 2:
                atoms.set_distance(total_atoms-8, i1, 1.3540, fix=1)
            else:
                atoms.set_distance(total_atoms-8, i1, 1.3390, fix=1)
        CnearH = near(total_atoms-8, atoms, 'H')
        del atoms[CnearH]
    elif place == 3:
        elements = atoms.get_chemical_symbols()
        elements[-6] = 'N'
        atoms.set_chemical_symbols(elements)
        for k in range(10):
            atoms.set_distance(total_atoms-6, total_atoms-8, 1.3390, fix=1)
            atoms.set_distance(total_atoms-6, total_atoms-5, 1.3390, fix=1)
        CnearH = near(total_atoms-6, atoms, 'H')
        del atoms[CnearH]
    elif place == 4:
        elements = atoms.get_chemical_symbols()
        elements[-5] = 'N'
        atoms.set_chemical_symbols(elements)
        for k in range(10):
            atoms.set_distance(total_atoms-5, total_atoms-6, 1.3390, fix=1)
            atoms.set_distance(total_atoms-5, total_atoms-7, 1.3390, fix=1)
        CnearH = near(total_atoms-5, atoms, 'H')
        del atoms[CnearH]
    elif place == 5:
        elements = atoms.get_chemical_symbols()
        elements[-7] = 'N'
        atoms.set_chemical_symbols(elements)
        for k in range(10):
            if state[0] == 2:
                atoms.set_distance(total_atoms-7, i2, 1.3540, fix=1)
            else:
                atoms.set_distance(total_atoms-7, i2, 1.3390, fix=1)
            atoms.set_distance(total_atoms-7, total_atoms-5, 1.3390, fix=1)
        CnearH = near(total_atoms-7, atoms, 'H')
        del atoms[CnearH]
    return atoms
        
def numjudge(i, deleted):
    if deleted<i:
        return i-1
    else:
        return i

def bondjudge2(Cnears, atoms, iis, elements, state, mask, times=10):
    print(Cnears, iis, elements)
    if 'N' in elements:
        for k in range(times):
            if state[0] == state[1] and state[0] == 2:
                atoms.set_distance(iis[0], Cnears[0], 1.5095, fix=1, mask=mask)
                atoms.set_distance(iis[1], Cnears[1], 1.3400, fix=1, mask=mask)
                atoms.set_distance(iis[0], iis[1], 1.2900, fix=1, mask=mask)
        return 7
    for k in range(times):
        if state[0] == state[1] and state[0] == 2:
            atoms.set_distance(iis[0], Cnears[0], 1.4000, fix=1, mask=mask)
            atoms.set_distance(iis[1], Cnears[1], 1.4000, fix=1, mask=mask)
            atoms.set_distance(iis[0], iis[1], 1.4000, fix=1, mask=mask)
        else:
            for i in range(2):
                if state[i] == 1:
                    atoms.set_distance(iis[i], Cnears[i], 1.5095, fix=1, mask=mask)
                elif state[i] == 2:
                    atoms.set_distance(iis[i], Cnears[i], 1.4600, fix=1, mask=mask)
                elif state[i] == 7:
                    atoms.set_distance(iis[i], Cnears[i], 1.3400, fix=1, mask=mask)
                elif state[i] == 8:
                    atoms.set_distance(iis[i], Cnears[i], 1.3640, fix=1, mask=mask)
                elif state[i] == 16:
                    atoms.set_distance(iis[i], Cnears[i], 1.7700, fix=1, mask=mask)
                elif state[i] == 82:
                    atoms.set_distance(iis[i], Cnears[i], 1.4440, fix=1, mask=mask)
            atoms.set_distance(iis[0], iis[1], 1.3400, fix=1, mask=mask)

def bondjudge5(Cnears, atoms, iis, state, mask, place, times=10):
    if place == 2:
        for k in range(times):
            if state[0] == state[1] and state[0] == 1:
                atoms.set_distance(iis[0], Cnears[0], 1.5040, fix=1, mask=mask)
                atoms.set_distance(iis[1], Cnears[1], 1.4950, fix=1, mask=mask)
                atoms.set_distance(i1, i2, 1.3670, fix=1, mask=mask)
            elif state[0] == state[1] and state[0] == 2:
                atoms.set_distance(iis[0], Cnears[0], 1.4040, fix=1, mask=mask)
                atoms.set_distance(iis[1], Cnears[1], 1.4000, fix=1, mask=mask)
                atoms.set_distance(i1, i2, 1.4240, fix=1, mask=mask)
    elif place == 3:
        for k in range(times):
            if state[0] == state[1] and state[0] == 2:
                atoms.set_distance(iis[0], Cnears[0], 1.4000, fix=1, mask=mask)
                atoms.set_distance(iis[1], Cnears[1], 1.4000, fix=1, mask=mask)
            else:
                for i in range(2):
                    if state[i] == 1:
                        atoms.set_distance(iis[i], Cnears[i], 1.4950, fix=1, mask=mask)
                    elif state[i] == 2:
                        atoms.set_distance(iis[i], Cnears[i], 1.4330, fix=1, mask=mask)
                    elif state[i] == 7:
                        atoms.set_distance(iis[i], Cnears[i], 1.4950, fix=1, mask=mask)
                    elif state[i] == 8:
                        atoms.set_distance(iis[i], Cnears[i], 1.3640, fix=1, mask=mask)
                    elif state[i] == 16:
                        atoms.set_distance(iis[i], Cnears[i], 1.7700, fix=1, mask=mask)
                    #elif state[i] == 82:
                    #    atoms.set_distance(iis[i], Cnears[i], 1.4440?, fix=1, mask=mask)
            atoms.set_distance(i1, i2, 1.4240, fix=1, mask=mask)
    elif place == 4:
        for k in range(times):
            if state[0] == state[1] and state[0] == 1:
                atoms.set_distance(iis[0], Cnears[0], 1.4950, fix=1, mask=mask)
                atoms.set_distance(iis[1], Cnears[1], 1.5040, fix=1, mask=mask)
            elif state[0] == state[1] and state[0] == 2:
                atoms.set_distance(iis[0], Cnears[0], 1.4000, fix=1, mask=mask)
                atoms.set_distance(iis[1], Cnears[1], 1.4040, fix=1, mask=mask)
            atoms.set_distance(i1, i2, 1.4240, fix=1, mask=mask)

def bondjudge6(Cnears, atoms, iis, state, mask, place, times=10):
    if place == 2 or place == 5:
        for k in range(times):
            if state[0] == state[1] and state[0] == 1:
                atoms.set_distance(iis[0], Cnears[0], 1.5100, fix=1, mask=mask)
                atoms.set_distance(iis[1], Cnears[1], 1.5100, fix=1, mask=mask)
                atoms.set_distance(i1, i2, 1.4000, fix=1, mask=mask)
            elif state[0] == state[1] and state[0] == 2:
                atoms.set_distance(iis[0], Cnears[0], 1.4040, fix=1, mask=mask)
                atoms.set_distance(iis[1], Cnears[1], 1.4040, fix=1, mask=mask)
                atoms.set_distance(i1, i2, 1.3700, fix=1, mask=mask)
    elif place == 1 or place == 3 or place == 4:
        for k in range(times):
            if state[0] == state[1] and state[0] == 2:
                atoms.set_distance(iis[0], Cnears[0], 1.4040, fix=1, mask=mask)
                atoms.set_distance(iis[1], Cnears[1], 1.4040, fix=1, mask=mask)
                atoms.set_distance(i1, i2, 1.3700, fix=1, mask=mask)
            elif state[0] == state[1] and state[0] == 82:
                atoms.set_distance(iis[0], Cnears[0], 1.4190, fix=1, mask=mask)
                atoms.set_distance(iis[1], Cnears[1], 1.4190, fix=1, mask=mask)
                atoms.set_distance(i1, i2, 1.3700, fix=1, mask=mask)
            else:
                for i in range(2):
                    if state[i] == 1:
                        atoms.set_distance(iis[i], Cnears[i], 1.5105, fix=1, mask=mask)
                    elif state[i] == 2:
                        atoms.set_distance(iis[i], Cnears[i], 1.4330, fix=1, mask=mask)
                    elif state[i] == 7:
                        atoms.set_distance(iis[i], Cnears[i], 1.3400, fix=1, mask=mask)
                    elif state[i] == 8:
                        atoms.set_distance(iis[i], Cnears[i], 1.3640, fix=1, mask=mask)
                    elif state[i] == 16:
                        atoms.set_distance(iis[i], Cnears[i], 1.7700, fix=1, mask=mask)
                    elif state[i] == 82:
                        atoms.set_distance(iis[i], Cnears[i], 1.4190, fix=1, mask=mask)
                    if state[i] == 2:
                        atoms.set_distance(i1, i2, 1.4040, fix=1, mask=mask)
                    else:
                        atoms.set_distance(i1, i2, 1.4000, fix=1, mask=mask)


def near(i, atoms, target='ALL', ignore=[], limit=True, reverse=False):
    elements = atoms.get_chemical_symbols()
    total_atoms = len(atoms)
    j = 0
    targeted_distances = []
    for element in elements:
        if j in ignore:
            j = j + 1
            continue
        if element == target:
            d = atoms.get_distances(i, j)
            if limit == True:
                if d[0] >= 0.5:
                    targeted_distances.append(d[0])
            else:
                targeted_distances.append(d[0])
        if target == 'Not H' or target == 'notH' or target == 'not H':
            if element != 'H':
                d = atoms.get_distances(i, j)
                if limit == True:
                    if d[0] >= 0.5:
                        targeted_distances.append(d[0])
                else:
                    targeted_distances.append(d[0])
        j = j + 1
    if reverse == True:
        _near = -1
    else:
        _near = 0
    if target == 'all' or target == 'All' or target == 'ALL':
        targeted_distances = atoms.get_distances(i, range(len(atoms)))
        targeted_distances = np.delete(targeted_distances,\
                                       np.where(targeted_distances == 0.0))
    distances = atoms.get_distances(i, range(len(atoms)))
    #print(distances, targeted_distances)
    for distance in distances:
        if total_atoms+_near in ignore:
            continue
        if reverse == True:
            if total_atoms+_near in ignore:
                _near = _near - 1
                continue
            if abs(min(targeted_distances)-atoms.get_distances(i,\
                                                               total_atoms+_near)) <= 0.001:
                #print(_near, atoms.get_distances(i, _near))
                return total_atoms+_near
            _near = _near - 1
        else:
            if _near in ignore:
                _near = _near + 1
                continue
            if abs(min(targeted_distances)-atoms.get_distances(i, _near)) <= 0.001:
                #print(_near, atoms.get_distances(i, _near))
                return _near
            #print(min(targeted_distances), atoms.get_distances(i, _near))
            _near = _near + 1


import re
import numpy as np
import pandas as pd
from sklearn.metrics import r2_score
from sklearn.linear_model import LinearRegression

def YModulus(file):
    '''
    Try to calculate the Young's Modulus based on LinearRegression.
    YModulus(file)
    file: File Name
    Unit: nN
    Use the calculated .txt file from sscurve.single or sscurve.multiple to get Young's Modulus.
    '''
    datum = pd.read_csv(file)
    b = re.search(r'[A-Z]+[a-z]*[A-Z]*[a-z]*[0-9]*\-?[A-Z]*[0-9]*_', file)
    #print(b.group(0)[:-1])
    datum = datum.rename(columns = {
        'Strain Length(%)':'Strain Length ({0})'.format(b.group(0)[:-1])})
    data = datum['Strain Length ({0})'.format(
        b.group(0)[:-1])].to_numpy().reshape(-1, 1)
    target = datum['Stress Force(nN)'].to_numpy().reshape(-1, 1)
    j = 3
    while 1:
        data2 = data[1:j]
        target2 = target[1:j]
        model = LinearRegression()
        model.fit(data2, target2)
        y = model.coef_[0][0]*data2 + model.intercept_
        test_score = r2_score(target2, y)
        #print(model.coef_[0][0], model.intercept_[0], test_score)
        if test_score < 0.99:
            break
        k = model.coef_[0][0]
        b = model.intercept_[0]
        j = j + 1
    return k


from MCPoly.orcaset import XyzToInp
from MCPoly.orcaset import orca
from MCPoly.orcaset import ssorca
from MCPoly.orcaset import multiorca
import os
import re
import pytest

def test_XyzToInp2():
    opath = os.getcwd()
    os.chdir('./MCPoly/tests')
    loc = './data_orcaset/'
    file = 'Atoms1'
    XyzToInp(file, fileloc=loc, saveloc=loc, method='B3LYP', basis_set='def2-TZVP', freq=True,\
            external_force=True, aim=[0, 3], strain=1.5, maxcore=4096, corenum=7)
    i1 = 0
    i2 = 0
    i3 = 0
    i4 = 0
    i5 = 0
    f = open(loc+file+'.inp', 'r')
    for line in f:
        a = re.search(r'B3LYP def2-TZVP OPT FREQ', line)
        if a:
            i1 = 1
        b = re.search(r'maxcore 4096', line)
        if b:
            i2 = 1
        c = re.search('%PAL NPROCS 7 END', line)
        if c:
            i3 = 1
        d1 = re.search(r'POTENTIALS', line)
        if d1:
            i4 = 1
        d2 = re.search('{C 0 3 1.500}', line)
        if d2:
            i5 = 1
            break
    f.close()
    os.chdir(opath)
    assert i1 * i2 * i3 * i4 * i5 != 0

def test_XyzToInp3():
    opath = os.getcwd()
    os.chdir('./MCPoly/tests')
    file = 'Atoms1'
    loc = './data_orcaset/'
    XyzToInp(file, fileloc=loc, saveloc=loc, method='B3LYP', basis_set='def2-TZVP', scan = True, \
            aim=[0, 3], stretch=0.8, scanstep=8, maxcore=4096, corenum=8)
    i1 = 0
    i2 = 0
    i3 = 0
    i4 = 0
    i5 = 0
    f = open(loc+file+'.inp', 'r')
    for line in f:
        a = re.search(r'B3LYP def2-TZVP', line)
        if a:
            i1 = 1
        b = re.search(r'maxcore 4096', line)
        if b:
            i2 = 1
        c = re.search('%PAL NPROCS 8 END', line)
        if c:
            i3 = 1
        d1 = re.search(r'geom Scan', line)
        if d1:
            i4 = 1
        d2 = re.search('B 0 3 = 3.804,  4.604,  8', line)
        if d2:
            i5 = 1
            break
    f.close()
    os.chdir(opath)
    assert i1 * i2 * i3 * i4 * i5 != 0

def test_XyzToInp4():
    opath = os.getcwd()
    os.chdir('./MCPoly/tests')
    file = 'Atoms1'
    loc = './data_orcaset/'
    XyzToInp(file, fileloc=loc, saveloc=loc, method='B3LYP D3BJ', basis_set='def2-TZVP',\
                     MD=True, freq=True, md_timestep=1.0, initvel=350,\
             tsNHC=350, dumpcon=25, runstep=2500, maxcore=4096, corenum=8)
    i1 = 0
    i2 = 0
    i3 = 0
    i4 = 0
    i5 = 0
    i6 = 0
    f = open(loc+file+'.inp', 'r')
    for line in f:
        print(line)
        a = re.search(r'MD B3LYP D3BJ def2-TZVP', line)
        if a:
            i1 = 1
        b = re.search(r'maxcore 4096', line)
        if b:
            i2 = 1
        c = re.search('%PAL NPROCS 8 END', line)
        if c:
            i3 = 1
        d1 = re.search(r'Timestep 1.0_fs', line)
        if d1:
            i4 = 1
        d2 = re.search('Dump Position Stride 25 Filename "Atoms1_dump.xyz"', line)
        if d2:
            i5 = 1
        d3 = re.search('Run 2500', line)
        if d3:
            i6 = 1
    f.close()
    os.chdir(opath)
    assert i1 * i2 * i3 * i4 * i5 * i6 != 0

def test_XyzToInp1():
    opath = os.getcwd()
    os.chdir('./MCPoly/tests')
    file = 'Atoms1'
    loc = './data_orcaset/'
    XyzToInp(file, fileloc = loc, saveloc = loc)
    i = 0
    j = 0
    f = open(loc+file+'.inp', 'r')
    for line in f:
        a = re.search(r'B3LYP def2-SVP', line)
        if a:
            i = 1
        b = re.search(r'xyz 0 1', line)
        if b:
            j = 1
            break
    f.close()
    os.chdir(opath)
    assert i * j != 0

def test_ORCA():
    opath = os.getcwd()
    os.chdir('./MCPoly/tests')
    file = 'Atoms1'
    loc = './data_orcaset/'
    orca(file, fileloc=loc, saveloc=loc)
    os.chdir(opath)
    return 0

@pytest.fixture
def suite():
    return multiorca(['Atoms1', 'Atoms2'], loc = './data_orcaset/')

def test_multiorca(suite):
    opath = os.getcwd()
    os.chdir('./MCPoly/tests')
    suite.run()
    os.chdir(opath)
    return 0

def object_ssorca():
    opath = os.getcwd()
    os.chdir('./MCPoly/tests')
    ssorca('Atoms3', loc = './data_orcaset/')
    os.chdir(opath)
    return 0

#def test_ssORCA(object_ssorca):
#    return ssorca(file3, loc=loc)


from ase.io import read
from ase import Atoms
import math as m

def straight(atoms, start, end):
    pos = atoms.get_positions()
    Apos = pos[start]
    Bpos = pos[end]
    atoms.rotate(m.atan((Bpos[-1]-Apos[-1])/(Bpos[1]-Apos[1]))/m.pi*180,\
                      'x', Apos)
    pos = atoms.get_positions()
    Apos = pos[start]
    Bpos = pos[end]
    atoms.rotate(m.atan((Bpos[-1]-Apos[-1])/(Bpos[0]-Apos[0]))/m.pi*180,\
                      'y', Apos)
    pos = atoms.get_positions()
    Apos = pos[start]
    Bpos = pos[end]
    atoms.rotate(m.atan(-(Bpos[1]-Apos[1])/(Bpos[0]-Apos[0]))/m.pi*180+180,\
                      'z', Apos)
    pos = atoms.get_positions()
    return atoms


import os
import re
import sys
import time as t
import ipywidgets as widgets
from IPython.display import display

mydir = os.path.dirname( __file__ )
orcadir = os.path.join(mydir, '..', 'orcaset')
sys.path.append(orcadir)
from XyzToInp import XyzToInp
from orca import orca

mydir = os.path.dirname( __file__ )
statusdir = os.path.join(mydir, '..', 'status')
sys.path.append(statusdir)
from status import status

mydir = os.path.dirname( __file__ )
viewdir = os.path.join(mydir, '..', 'view3d')
sys.path.append(viewdir)
from view3d import view3d

def mgui():
    """
        The method to use GUI to set ORCA input files and calculate multiple files consecutively by ORCA.
        You will know how to do as long as you get into the GUI platform.

        TIPS: 1. To see the status of your optimisation,  you can set another window and use status.figuretraj .
              2. This visualisation will greatly restrict the flexibility of the task,  to make you programme more flexible,  please directly use the code.
    """

    print('TIPS: Multiple values can be selected with Shift and/or Ctrl (or Command) pressed and mouse clicks or arrow keys.')
    
    output = widgets.Output()
    output2 = widgets.Output()
    scanoutput = widgets.Output()
    forceoutput = widgets.Output()
    eoutput = widgets.Output()
    geooutput = widgets.Output()
    case = widgets.Output()
    
    def changecombo(loc):
        options2 = []
        for path in os.listdir(loc):
            if os.path.isfile(os.path.join(loc, path)):
                a = re.search('.out', path)
                if a:
                    options2.append(path[:-4])
        options2.sort()
        files.options = options2
    
    def geoshow(files, loc, method, bs):
        output.clear_output()
        with output:
            if files == []:
                print('Please input the file to show the geometry structure on this screen.')
            for file in files:
                try:
                    print('File: {0}.xyz'.format(file))
                    view3d(file, loc, width=450, height=400)
                except:
                    print("'{0}.xyz' is not found.".format(file))
            if method != '' and bs != '':
                display(widgets.HBox([widgets.Label('ORCA Location:'),\
                                      orcaloc]), startorca)
    
    def intro_status3d(button):
        ii = 0
        for file in files.value:
            ii = ii + 1
            if roomox.value == False and coreox.value == False:
                XyzToInp(file, fileloc=loc.value, method=method.value,\
                         basis_set=bs.value, opt=opt.value, freq=freq.value,\
                         maxiter=maxiter.value, maxcore=room.value*1024,\
                         corenum=corenum.value, electron=e.value, state=state.value)
            elif roomox.value == False and coreox.value == True:
                XyzToInp(file, fileloc=loc.value, method=method.value,\
                         basis_set=bs.value, opt=opt.value, freq=freq.value,\
                         maxiter=maxiter.value, maxcore=room.value*1024,\
                         electron=e.value, state=state.value)
            elif roomox.value == True and coreox.value == False:
                XyzToInp(file, fileloc=loc.value, method=method.value,\
                         basis_set=bs.value, opt=opt.value, freq=freq.value,\
                         maxiter=maxiter.value, corenum=corenum.value,\
                         electron=e.value, state=state.value)
            elif roomox.value == True and coreox.value == True:
                XyzToInp(file, fileloc=loc.value, method=method.value,\
                         basis_set=bs.value, opt=opt.value, freq=freq.value,\
                         maxiter=maxiter.value, electron=e.value, state=state.value)
            with case:
                display(widgets.HBox([widgets.Label(file), widgets.Label('  Processing...'),\
                                      widgets.Label('[{0}]'.format(t.ctime(t.time())))]))
            orca(file, orcaloc.value, loc.value, loc.value)
            s = status(file, loc.value).status(statusonly=True)
            if s == 4:
                case.clear_output()
                with case:
                    display(widgets.Valid(description = '     ' + file, value=False, \
                            readout = 'Retrying...'),\
                            widgets.Label('[{0}]'.format(t.ctime(t.time()))))
                orca(file, orcaloc.value, loc.value, loc.value)
                s = status(file, loc.value).status(statusonly=True)
                if s == 4:
                    case.clear_output()
                    with output2:
                        display(widgets.Valid(description = '     ' + file, value=False, \
                                readout = 'Error!'),\
                                widgets.Label('[{0}]'.format(t.ctime(t.time()))))
            elif s == 2:
                case.clear_output()
                with output2:
                    display(widgets.HBox([widgets.Valid(description = '     ' + file,\
                                                        value=True),\
                                          widgets.Label('Completed.'),\
                                          widgets.Label('[{0}]'.format(t.ctime(t.time())))]))
            if ii == 1:
                output.clear_output()
            with output:
                print('File: {0}.xyz    Energy: {1:.6f} Hartree'.format(file,\
                                                                        status(file, loc.value).status(figureonly = True)[-1]))
                view3d(file, loc.value, width=450, height=400)
    
    options = []
    loc = widgets.Text(description = 'Location:', value='./')
    for path in os.listdir(loc.value):
        if os.path.isfile(os.path.join(loc.value,  path)):
            a = re.search('.xyz', path)
            if a:
                b = re.search('_trj', path)
                if b:
                    pass
                else:
                    options.append(path[:-4])
    options.sort()
    files = widgets.SelectMultiple(options=options, description = 'xyz Files:')
    method = widgets.Text(description = 'Method:')
    bs = widgets.Text(description = 'Basis Set:')
    opt = widgets.ToggleButton(value=False, description = 'Optimisation',\
                               layout = widgets.Layout(width='50%'))
    freq = widgets.ToggleButton(value=False, description = 'Frequency',\
                                layout = widgets.Layout(width='50%'))
    room = widgets.IntText(description = 'Space(GB):', value=4, min=1,\
                           layout = widgets.Layout(width='50%'))
    roomox = widgets.ToggleButton(value=False, description = 'Default Space')
    corenum = widgets.IntText(description = 'Core:', value=4, min=1,\
                              layout = widgets.Layout(width='50%'))
    coreox = widgets.ToggleButton(value=False, description = 'Default CoreNumber')
    e = widgets.IntText(description = 'Charge:', value=0,\
                        layout = widgets.Layout(width='50%'))
    state = widgets.IntText(description = 'State:', value=1,\
                            layout = widgets.Layout(width='47.5%'))
    geoox = widgets.ToggleButton(value=False, description = 'Structure Status',\
                                 style = dict(font_weight='bold'), layout = widgets.Layout(width='98.5%'))
    startorca = widgets.Button(description = 'Start ORCA Calculation',\
                               layout = widgets.Layout(width='99%'),\
                               style = dict(font_weight = 'bold', text_decoration = 'underline'))
    
    changecomboout = widgets.interactive_output(changecombo, {'loc': loc})
    geoout = widgets.interactive_output(geoshow, {'files': files, 'loc': loc,\
                                                  'method': method, 'bs': bs})
    orcaloc = widgets.Text(value = './')
    maxiter = widgets.IntText(description = 'MaxIteration:', value=200,\
                              layout = widgets.Layout(width='98.5%'))
    
    title1 = widgets.Label('File Details', style = dict(font_weight='bold'))
    title2 = widgets.Label('Calculation Methods',\
                           style = dict(font_weight='bold'))
    
    box_layout1  =  widgets.Layout(display='flex', flew_flow='column',\
                                   overflow = 'scroll hidden', align_items = 'stretch',\
                                   border='solid',  height='650px',  width='31%')
    box_layout2  =  widgets.Layout(display='flex',  flew_flow='column',\
                                   overflow = 'scroll hidden', align_items = 'stretch',\
                                   border='solid',  height='650px',  width='33%')
    widgets.jslink((roomox, 'value'), (room, 'disabled'))
    widgets.jslink((coreox, 'value'), (corenum, 'disabled'))
    #introstatus = widgets.Button(description = 'See Current Calculation Status',\
    #                             style = dict(font_weight = 'bold', text_decoration = 'underline'), 
    #                  layout = widgets.Layout(width='98.5%'))

    frame1 = widgets.VBox([title1, loc, files, title2, widgets.HBox([opt, freq]),\
                           method, bs, widgets.HBox([corenum, coreox]), \
                           widgets.HBox([room, roomox]), maxiter, widgets.HBox([e, state])],\
                          layout = box_layout1)
    frame3 = widgets.VBox([output2, case])
    output.layout = box_layout2
    frame3.layout = box_layout2
    startorca.on_click(intro_status3d)

    display(widgets.HBox([frame1, output, frame3]))


import sys
import os
import ipywidgets as widgets

mydir = os.path.dirname( __file__ )
statusdir = os.path.join(mydir, '..', 'status')
sys.path.append(statusdir)
from status import status
from gui import gui as gui3

mydir = os.path.dirname( __file__ )
orcadir = os.path.join(mydir, '..')#, 'orcaset')
sys.path.append(orcadir)
from orcaset.gui import gui as gui1
from sscurve.gui import gui as gui2
from moldraw.gui import gui as gui4
from sscurve.gui2 import gui2 as gui5

mydir = os.path.dirname( __file__ )
orcadir = os.path.join(mydir, '..', 'orcaset')
sys.path.append(orcadir)
from ssgui import ssgui

mydir = os.path.dirname( __file__ )
orcadir = os.path.join(mydir, '..', 'orcaset')
sys.path.append(orcadir)
from mgui import mgui

#mydir = os.path.dirname( __file__ )
#ssdir = os.path.join(mydir, '..', 'sscurve')
#sys.path.append(ssdir)
#from gui import gui as gui2

def vis():
    """
        The method to summon all four GUI functions.
        You can see how to use by input 'MCPoly.orcaset.gui?'(Normal ORCA), 'MCPoly.orcaset.ssgui?' (Mechanical ORCA), 'MCPoly.orcaset.mgui?' (Multiple ORCA) and 'MCPoly.sscurve.gui?' (Stress-Strain Curve)
    """
    screen1 = widgets.Output()
    screen2 = widgets.Output()
    screen3 = widgets.Output()
    screen4 = widgets.Output()
    screen5 = widgets.Output()
    screen6 = widgets.Output()
    screen7 = widgets.Output()
    
    with screen1:
        gui1()
    with screen2:
        ssgui()
    with screen3:
        mgui()
    with screen4:
        gui2()
    with screen5:
        gui5()
    with screen6:
        gui3()
    with screen7:
        gui4()
    
    tab_contents = ['Normal ORCA', 'Mechanical ORCA', 'Multiple ORCA', 'Stress-Strain Curve', 'Distance Curve', 'Energy Diagram', 'Molecule Designer']
    tab = widgets.Tab(style=dict(font_weight='bold'))
    #children = [widgets.Label(name,\
    #            style=dict(font_weight='bold')) for name in tab_contents]
    children = [screen1, screen2, screen3, screen4, screen5, screen6, screen7]
    tab.children = children
    tab.titles = tab_contents
    tab.layout = widgets.Layout(layout=widgets.Layout(width='50%'))
    display(tab)


import os
import sys
import numpy as np
from ase.io import read
from ase import Atoms
import math as m

mydir = os.path.dirname( __file__ )
statusdir = os.path.join(mydir, '..', 'status')
sys.path.append(statusdir)
from status import status

def untie(file1, file2, tolerance=0.015, loc='./', savefile=''):
    """
    A method to see the relative element number of the same conformer after being disordered, and you can also save the relevent file into .xyz format.
    untie(file1, file2, tolerance=0.015, num=-1, loc='./', savefile='')
    file1, file2: File Name.
    tolerance: If some bonds, angles or dihedrals has small changes, you can change it to get better results. The default is 0.015 Å.
    loc: File Location. The default is your current location.
    savefile: The name of the saved file.
    TIPS: Pay attention to tolerance. If it's too small, the result will not show the full result. If it's too big, the result will be inaccurate.
    
    Example:
        Input:
            from MCPoly.moldraw import untie
            result = untie(file1, file2, tolerance=0.015, num=-1, loc='./', savefile='')
            print(result)
        
        Output:
            {0: 0, 1: 15, 2: 16, 3: 17, 4: 18, 5: 19, 6: 20, 7: 21, 8: 22,\
             9: 23, 10: 24, 11: 1, 12: 2, 13: 3, 14: 4, 15: 5, 16: 6, 17: 7,\
             18: 8, 19: 9, 20: 10, 21: 11, 22: 12, 23: 13, 24: 14}
    """
    
    path = os.getcwd()
    os.chdir(loc)
    
    atoms1 = read(file1 + '.xyz')
    atoms2 = read(file2 + '.xyz')
    
    targeted_distances = atoms1.get_distances(0,range(len(atoms1)))
    targeted_distances = np.delete(targeted_distances,\
                                   np.where(targeted_distances <= 1.2))
    elements = atoms1.get_chemical_symbols()
    elements_not_H1 = []
    edge1 = 0
    max_element1 = 0
    max_distances1 = []
    max_all_distances = 0
    
    for i, element in enumerate(elements):
        if element != 'H':
            elements_not_H1.append(i)
    for i in elements_not_H1:
        distances_not_H1 = atoms1.get_distances(i, elements_not_H1)
        all_distances_not_H1 = np.sum(distances_not_H1)
        if all_distances_not_H1 > max_all_distances:
            max_element1 = i
            max_distances1 = distances_not_H1
            max_all_distances = all_distances_not_H1
    for i, distance in enumerate(max_distances1):
        if distance == np.max(max_distances1):
            max_num1 = elements_not_H1[i]
            break
    targeted_distances1 = atoms1.get_distances(max_element1, range(len(atoms1)))

    targeted_distances = atoms2.get_distances(0, range(len(atoms2)))
    targeted_distances = np.delete(targeted_distances,\
                                   np.where(targeted_distances <= 1.2))
    elements = atoms2.get_chemical_symbols()
    elements_not_H2 = []
    edge2 = 0
    max_element2 = 0
    max_distances2 = []
    max_all_distances = 0
    
    for i, element in enumerate(elements):
        if element != 'H':
            elements_not_H2.append(i)
    for i in elements_not_H2:
        distances_not_H2 = atoms2.get_distances(i, elements_not_H2)
        all_distances_not_H2 = np.sum(distances_not_H2)
        if all_distances_not_H2 > max_all_distances:
            max_element2 = i
            max_distances2 = distances_not_H2
            max_all_distances = all_distances_not_H2
    for i, distance in enumerate(max_distances2):
        if distance == np.max(max_distances2):
            max_num2 = elements_not_H2[i]
            break
    targeted_distances2 = atoms2.get_distances(max_element2, range(len(atoms2)))

    result = {}
    for i, distance1 in enumerate(targeted_distances1):
        for j, distance2 in enumerate(targeted_distances2):
            if abs(distance1 - distance2) <= tolerance:
                if j in result.values():
                    continue
                result[i] = j
                break
            result[i] = None
    
    status2 = status(file2).atom()
    
    try:
        if savefile == '':
            f = open('{0}_untied.xyz'.format(file2), 'x')
        else:
            f = open('{0}.xyz'.format(savefile), 'x')
    except:
        if savefile == '':
            f = open('{0}_untied.xyz'.format(file2), 'w')
        else:
            f = open('{0}.xyz'.format(savefile), 'w')
    f.write('{0}\n\n'.format(len(status2)))
    for i in range(len(status2)):
        j = result[i]
        f.write('{0} {1:.5f} {2:.5f} {3:.5f}\n'.format(*status2[j]))
    f.close()
    
    os.chdir(path)
    
    return result


import re
import os
import warnings
import math as m
from ase.io import read,  write

def infchain(file, bondtype, loc = './'):
    """
    The method to create a single molecule for periodical chain, specialised for polymers.
    infchain(file, bondtype, loc = './')
    file: Your molecule system name on your .data file.
    bondtype: The bond type between the start and the end of the polymer. 
    loc: File Location. The default is your current location.
    Example:
        Input:
            from MCPoly.lmpset import chain
            infchain('Poly1', 11)
        Output in Poly1_Chain.data:
            82 atoms
            84 bonds
            154 angles
            211 dihedrals
            47 impropers
            
            9 atom types
            12 bond types
            21 angle types
            31 dihedral types
            3 improper types
            
            ...
            84 11 79 1
            
            ...
    """
    opath = os.getcwd()
    os.chdir(loc)
    k = 0
    w1 = 0
    w2 = 0
    w3 = 0
    w4 = 0
    w5 = 0
    n1 = 0
    t2 = 0
    t3 = 0
    t4 = 0
    t5 = 0
    H = []
    l = []
    real = [0, 0, 0]
    inp  =  open('{0}.data'.format(file), 'r')
    for line in inp:
        a1 = re.search(r' atoms', line)
        a2 = re.search(r' bonds', line)
        a3 = re.search(r' angles', line)
        a4 = re.search(r' dihedrals', line)
        a5 = re.search(r' impropers', line)
        m1 = re.search(r'Masses', line)
        if a1:
            b1 = re.search(r'[0-9]+', line)
            num = b1.group(0)
        elif a2:
            b2 = re.search(r'[0-9]+', line)
            num2 = b2.group(0)
        elif a3:
            b3 = re.search(r'[0-9]+', line)
            num3 = b3.group(0)
        elif a4:
            b4 = re.search(r'[0-9]+', line)
            num4 = b4.group(0)
        elif a5:
            b5 = re.search(r'[0-9]+', line)
            num5 = b5.group(0)
        if n1 == 1:
            l1 = re.findall(r'\-?[0-9]+\.?[0-9]*', line)
            if l1 == []:
                continue
            else:
                n1 = 2
        if n1 == 2:
            l1 = re.findall(r'\-?[0-9]+\.?[0-9]*', line)
            if l1 == []:
                n1 = 0
                continue
            if eval(l1[-1]) < 1.1:
                H.append(eval(l1[0]))
        if m1:
            n1 = 1
        c1 = re.search('Atoms', line)
        c2 = re.search('Bonds', line)
        c3 = re.search('Angles', line)
        c4 = re.search('Dihedrals', line)
        c5 = re.search('Impropers', line)
        if c1:
            w1 = 1
        elif c2:
            w2 = 1
        elif c3:
            w3 = 1
        elif c4:
            w4 = 1
        elif c5:
            w5 = 1
        elif w1 == 1:
            l1 = re.findall(r'\-?[0-9]+\.?[0-9]*', line)
            if len(l1) >= 6:
                l.append(l1)
                if l1[0] == num:
                    w1 = 2
        elif w1 == 2:
            xmin = 99999
            xmax = -99999
            xmaxnum = 0
            xminnum = 0
            realxmin = 99999
            realxmax = -99999
            realxmaxnum = 0
            realxminnum = 0
            for i4 in range(len(l)):
                if eval(l[i4][2]) not in H:
                    continue
                if eval(l[i4][-3]) > xmax:
                    xmax = eval(l[i4][-3])
                    xmaxnum = eval(l[i4][0])
                if eval(l[i4][-3]) < xmin:
                    xmin = eval(l[i4][-3])
                    xminnum = eval(l[i4][0])
            for i4 in range(len(l)):
                if eval(l[i4][2]) in H:
                    continue
                if eval(l[i4][-3]) > realxmax:
                    realxmax = eval(l[i4][-3])
                    realxmaxnum = eval(l[i4][0])
                if eval(l[i4][-3]) < realxmin:
                    realxmin = eval(l[i4][-3])
                    realxminnum = eval(l[i4][0])
            w1 = 0
        elif w2 == 1:
            l1 = re.findall(r'[0-9]+', line)
            if len(l1) == 4:
                l.append(l1)
                if l1[0] == num2:
                    w2 = 2
                    j = 1
        elif w2 == 2:
            for i4 in range(len(l)):
                if eval(l[i4][2]) in [xminnum, xmaxnum]:
                    continue
                if eval(l[i4][3]) in [xminnum, xmaxnum]:
                    continue
                j = j + 1
            k = k + 1
            l = []
            t2 = j
            j = 1
            w2 = 0
            k = 0
        elif w3 == 1:
            l1 = re.findall(r'[0-9]+', line)
            if len(l1) == 5:
                l.append(l1)
                if l1[0] == num3:
                    w3 = 2
                    j = 1
        elif w3 == 2:
            for i4 in range(len(l)):
                if eval(l[i4][2]) in [xminnum, xmaxnum]:
                    continue
                if eval(l[i4][3]) in [xminnum, xmaxnum]:
                    continue
                if eval(l[i4][4]) in [xminnum, xmaxnum]:
                    continue
                j = j+1
            k = k+1
            l = []
            t3 = j
            j = 1
            k = 0
            w3 = 0
        elif w4 == 1:
            l1 = re.findall(r'[0-9]+', line)
            if len(l1) == 6:
                l.append(l1)
                if l1[0] == num4:
                    w4 = 2
                    j = 1
        elif w4 == 2:
            for i4 in range(len(l)):
                if eval(l[i4][2]) in [xminnum, xmaxnum]:
                    continue
                if eval(l[i4][3]) in [xminnum, xmaxnum]:
                    continue
                if eval(l[i4][4]) in [xminnum, xmaxnum]:
                    continue
                if eval(l[i4][5]) in [xminnum, xmaxnum]:
                    continue
                j = j+1
            k = k+1
            l = []
            t4 = j
            j = 1
            k = 0
            w4 = 0
        elif w5 == 1:
            l1 = re.findall(r'[0-9]+', line)
            if len(l1) == 6:
                l.append(l1)
                if l1[0] == num5:
                    w5 = 2
                    j = 1
        elif w5 == 2:
            for i4 in range(len(l)):
                if eval(l[i4][2]) in [xminnum, xmaxnum]:
                    continue
                if eval(l[i4][3]) in [xminnum, xmaxnum]:
                    continue
                if eval(l[i4][4]) in [xminnum, xmaxnum]:
                    continue
                if eval(l[i4][5]) in [xminnum, xmaxnum]:
                    continue
                j = j + 1
            k = k + 1
            l = []
            t5 = j
            j = 1
            k = 0
            w4 = 0
    inp.close()
#    print(t2, t3, t4, t5)
    inp  =  open('{0}.data'.format(file), 'r')
    k = 0
    w1 = 0
    w2 = 0
    w3 = 0
    w4 = 0
    w5 = 0
    gap = 0
    l = []
    try:
        f  =  open('{0}_Chain.data'.format(file), 'x')
    except:
        f  =  open('{0}_Chain.data'.format(file), 'w')
    for line in inp:
        a1 = re.search(r' atoms', line)
        a2 = re.search(r' bonds', line)
        a3 = re.search(r' angles', line)
        a4 = re.search(r' dihedrals', line)
        a5 = re.search(r' impropers', line)
        a6 = re.search(r' xlo xhi', line)
        a7 = re.search(r' ylo yhi', line)
        a8 = re.search(r' zlo zhi', line)
        c1 = re.search('Atoms', line)
        c2 = re.search('Bonds', line)
        c3 = re.search('Angles', line)
        c4 = re.search('Dihedrals', line)
        c5 = re.search('Impropers', line)
        if a1:
            b1 = re.search(r'[0-9]+', line)
            f.write('{0} atoms\n'.format(eval(b1.group(0))-2))
            num = b1.group(0)
        elif a2:
            b2 = re.search(r'[0-9]+', line)
            f.write('{0} bonds\n'.format(eval(b2.group(0))-1))
            num2 = b2.group(0)
        elif a3:
            f.write('{0} angles\n'.format(t3-1))
            num3 = b3.group(0)
        elif a4:
            b4 = re.search(r'[0-9]+', line)
            f.write('{0} dihedrals\n'.format(t4-1))
            num4 = b4.group(0)
        elif a5:
            b5 = re.search(r'[0-9]+', line)
            f.write('{0} impropers\n'.format(t5-1))
            num5 = b5.group(0)
        elif a6:
            x1 = re.findall(r'\-?[0-9]+\.?[0-9]*', line)
            xd = eval(x1[1])-eval(x1[0])
            f.write('{0:.5f} {1:.5f} xlo xhi\n'.format(xmin, xmax))
        elif a7:
            y1 = re.findall(r'\-?[0-9]+\.?[0-9]*', line)
            yd = eval(y1[1])-eval(y1[0])
            f.write('{0:.5f} {1:.5f} ylo yhi\n'.format(eval(y1[0]), eval(y1[1])))
        elif a8:
            z1 = re.findall(r'\-?[0-9]+\.?[0-9]*', line)
            zd = eval(z1[1])-eval(z1[0])
            f.write('{0:.5f} {1:.5f} zlo zhi\n'.format(eval(z1[0]), eval(z1[1])))
        elif c1:
            w1 = 1
            f.write(line)
            f.write('\n')
        elif c2:
            w2 = 1
            f.write(line)
            f.write('\n')
        elif c3:
            w3 = 1
            f.write(line)
            f.write('\n')
        elif c4:
            w4 = 1
            f.write(line)
            f.write('\n')
        elif c5:
            w5 = 1
            f.write(line)
            f.write('\n')
        elif w1 == 1:
            l1 = re.findall(r'\-?[0-9]+\.?[0-9]*', line)
            if len(l1) >= 6:
                l.append(l1)
                if l1[0] == num:
                    w1 = 2
                    j = 1
        elif w1 == 2:
            for i4 in range(eval(b1.group(0))-2+1):
                if j in [xminnum, xmaxnum]:
                    gap = gap + 1
                    j = j + 1
                    continue
                f.write('{0} {1} {2} {3:.4f} {4:.5f} {5:.5f} {6:.5f}\n'.format(j-gap,\
                                                                               eval(l[i4][1]), eval(l[i4][2]), eval(l[i4][3]),\
                                                                               eval(l[i4][-3]), eval(l[i4][-2]), eval(l[i4][-1])))
                j = j + 1
            l = []
            j = 1
            w1 = 0
            f.write('\n')
        elif w2 == 1:
            l1 = re.findall(r'[0-9]+', line)
            if len(l1) == 4:
                l.append(l1)
                if l1[0] == num2:
                    w2 = 2
                    j = 1
        elif w2 == 2:
            for i4 in range(eval(b2.group(0))-1+1):
                if eval(l[i4][2]) in [xminnum, xmaxnum]:
                    continue
                elif eval(l[i4][2]) > min([xminnum, xmaxnum]):
                    l[i4][2] = str(eval(l[i4][2])-1)
                elif eval(l[i4][2]) > max([xminnum, xmaxnum]):
                    l[i4][2] = str(eval(l[i4][2])-2)  
                if eval(l[i4][3]) in [xminnum, xmaxnum]:
                    continue
                elif eval(l[i4][3]) > min([xminnum, xmaxnum]):
                    l[i4][3] = str(eval(l[i4][3])-1)
                elif eval(l[i4][3]) > max([xminnum, xmaxnum]):
                    l[i4][3] = str(eval(l[i4][3])-2)  
                f.write('{0} {1} {2} {3}\n'.format(j, eval(l[i4][1]),\
                                                   eval(l[i4][2]), eval(l[i4][3])))
                j = j + 1
            k = k + 1
            l = []
            if realxmaxnum in [xminnum, xmaxnum]:
                continue
            elif realxmaxnum > min([xminnum, xmaxnum]):
                realxmaxnum = realxmaxnum-1
            elif realxmaxnum > max([xminnum, xmaxnum]):
                realxmaxnum = realxmaxnum-2  
            if realxminnum in [xminnum, xmaxnum]:
                continue
            elif realxminnum > min([xminnum, xmaxnum]):
                realxminnum = realxminnum-1
            elif realxminnum > max([xminnum, xmaxnum]):
                realxminnum = realxminnum-2  
            f.write('{0} {1} {2} {3}\n'.format(j, bondtype,\
                                               realxmaxnum, realxminnum))
            j = 1
            w2 = 0
            k = 0
            f.write('\n')
        elif w3 == 1:
            l1 = re.findall(r'[0-9]+', line)
            if len(l1) == 5:
                l.append(l1)
                if l1[0] == num3:
                    w3 = 2
                    j = 1
        elif w3 == 2:
            for i4 in range(len(l)):
                if eval(l[i4][2]) in [xminnum, xmaxnum]:
                    continue
                elif eval(l[i4][2]) > min([xminnum, xmaxnum]):
                    l[i4][2] = str(eval(l[i4][2])-1)
                elif eval(l[i4][2]) > max([xminnum, xmaxnum]):
                    l[i4][2] = str(eval(l[i4][2])-2)  
                if eval(l[i4][3]) in [xminnum, xmaxnum]:
                    continue
                elif eval(l[i4][3]) > min([xminnum, xmaxnum]):
                    l[i4][3] = str(eval(l[i4][3])-1)
                elif eval(l[i4][3]) > max([xminnum, xmaxnum]):
                    l[i4][3] = str(eval(l[i4][3])-2)  
                if eval(l[i4][4]) in [xminnum, xmaxnum]:
                    continue
                elif eval(l[i4][4]) > min([xminnum, xmaxnum]):
                    l[i4][4] = str(eval(l[i4][4])-1)
                elif eval(l[i4][4]) > max([xminnum, xmaxnum]):
                    l[i4][4] = str(eval(l[i4][4])-2)  
                f.write('{0} {1} {2} {3} {4}\n'.format(j, eval(l[i4][1]),\
                                                       eval(l[i4][2]), eval(l[i4][3]), eval(l[i4][4])))
                j = j + 1
            k = k + 1
            l = []
            j = 1
            k = 0
            w3 = 0
            f.write('\n')
        elif w4 == 1:
            l1 = re.findall(r'[0-9]+', line)
            if len(l1) == 6:
                l.append(l1)
                if l1[0] == num4:
                    w4 = 2
                    j = 1
        elif w4 == 2:
            for i4 in range(len(l)):
                if eval(l[i4][2]) in [xminnum, xmaxnum]:
                    continue
                elif eval(l[i4][2]) > min([xminnum, xmaxnum]):
                    l[i4][2] = str(eval(l[i4][2])-1)
                elif eval(l[i4][2]) > max([xminnum, xmaxnum]):
                    l[i4][2] = str(eval(l[i4][2])-2)  
                if eval(l[i4][3]) in [xminnum, xmaxnum]:
                    continue
                elif eval(l[i4][3]) > min([xminnum, xmaxnum]):
                    l[i4][3] = str(eval(l[i4][3])-1)
                elif eval(l[i4][3]) > max([xminnum, xmaxnum]):
                    l[i4][3] = str(eval(l[i4][3])-2)  
                if eval(l[i4][4]) in [xminnum, xmaxnum]:
                    continue
                elif eval(l[i4][4]) > min([xminnum, xmaxnum]):
                    l[i4][4] = str(eval(l[i4][4])-1)
                elif eval(l[i4][4]) > max([xminnum, xmaxnum]):
                    l[i4][4] = str(eval(l[i4][4])-2)  
                if eval(l[i4][5]) in [xminnum, xmaxnum]:
                    continue
                elif eval(l[i4][5]) > min([xminnum, xmaxnum]):
                    l[i4][5] = str(eval(l[i4][5])-1)
                elif eval(l[i4][5]) > max([xminnum, xmaxnum]):
                    l[i4][5] = str(eval(l[i4][5])-2)
                f.write('{0} {1} {2} {3} {4} {5}\n'.format(j, eval(l[i4][1]),\
                                                           eval(l[i4][2]), eval(l[i4][3]), eval(l[i4][4]), eval(l[i4][5])))
                j = j + 1
            k = k + 1
            l = []
            j = 1
            k = 0
            w4 = 0
            f.write('\n')
        elif w5 == 1:
            l1 = re.findall(r'[0-9]+', line)
            if len(l1) == 6:
                l.append(l1)
                if l1[0] == num5:
                    w5 = 2
                    j = 1
        elif w5 == 2:
            for i4 in range(len(l)):
                if eval(l[i4][2]) in [xminnum, xmaxnum]:
                    continue
                elif eval(l[i4][2]) > min([xminnum, xmaxnum]):
                    l[i4][2] = str(eval(l[i4][2])-1)
                elif eval(l[i4][2]) > max([xminnum, xmaxnum]):
                    l[i4][2] = str(eval(l[i4][2])-2)  
                if eval(l[i4][3]) in [xminnum, xmaxnum]:
                    continue
                elif eval(l[i4][3]) > min([xminnum, xmaxnum]):
                    l[i4][3] = str(eval(l[i4][3])-1)
                elif eval(l[i4][3]) > max([xminnum, xmaxnum]):
                    l[i4][3] = str(eval(l[i4][3])-2)  
                if eval(l[i4][4]) in [xminnum, xmaxnum]:
                    continue
                elif eval(l[i4][4]) > min([xminnum, xmaxnum]):
                    l[i4][4] = str(eval(l[i4][4])-1)
                elif eval(l[i4][4]) > max([xminnum, xmaxnum]):
                    l[i4][4] = str(eval(l[i4][4])-2)  
                if eval(l[i4][5]) in [xminnum, xmaxnum]:
                    continue
                elif eval(l[i4][5]) > min([xminnum, xmaxnum]):
                    l[i4][5] = str(eval(l[i4][5])-1)
                elif eval(l[i4][5]) > max([xminnum, xmaxnum]):
                    l[i4][5] = str(eval(l[i4][5])-2)
                f.write('{0} {1} {2} {3} {4} {5}\n'.format(j, eval(l[i4][1]),\
                                                           eval(l[i4][2]), eval(l[i4][3]), eval(l[i4][4]), eval(l[i4][5])))
                j = j + 1
            k = k + 1
            l = []
            j = 1
            k = 0
            w4 = 0
            f.write('\n')
        else:
            f.write(line)
    f.close()
    inp.close()
    os.chdir(opath)